# File automatically generated using autopxd2

from libc.stdio cimport FILE

cdef extern from "unuran.h" nogil:

    cdef struct unur_distr

    ctypedef unur_distr UNUR_DISTR

    cdef struct unur_par

    ctypedef unur_par UNUR_PAR

    cdef struct unur_gen

    ctypedef unur_gen UNUR_GEN

    cdef struct unur_urng

    ctypedef unur_urng UNUR_URNG

    ctypedef double UNUR_FUNCT_CONT(double x, unur_distr* distr)

    ctypedef double UNUR_FUNCT_DISCR(int x, unur_distr* distr)

    ctypedef int UNUR_IFUNCT_DISCR(double x, unur_distr* distr)

    ctypedef double UNUR_FUNCT_CVEC(double* x, unur_distr* distr)

    ctypedef int UNUR_VFUNCT_CVEC(double* result, double* x, unur_distr* distr)

    ctypedef double UNUR_FUNCTD_CVEC(double* x, int coord, unur_distr* distr)

    cdef struct unur_slist

    ctypedef void UNUR_ERROR_HANDLER(char* objid, char* file, int line, char* errortype, int unur_errno, char* reason)

    UNUR_URNG* unur_get_default_urng()

    UNUR_URNG* unur_set_default_urng(UNUR_URNG* urng_new)

    UNUR_URNG* unur_set_default_urng_aux(UNUR_URNG* urng_new)

    UNUR_URNG* unur_get_default_urng_aux()

    int unur_set_urng(UNUR_PAR* parameters, UNUR_URNG* urng)

    UNUR_URNG* unur_chg_urng(UNUR_GEN* generator, UNUR_URNG* urng)

    UNUR_URNG* unur_get_urng(UNUR_GEN* generator)

    int unur_set_urng_aux(UNUR_PAR* parameters, UNUR_URNG* urng_aux)

    int unur_use_urng_aux_default(UNUR_PAR* parameters)

    int unur_chgto_urng_aux_default(UNUR_GEN* generator)

    UNUR_URNG* unur_chg_urng_aux(UNUR_GEN* generator, UNUR_URNG* urng_aux)

    UNUR_URNG* unur_get_urng_aux(UNUR_GEN* generator)

    double unur_urng_sample(UNUR_URNG* urng)

    double unur_sample_urng(UNUR_GEN* gen)

    int unur_urng_sample_array(UNUR_URNG* urng, double* X, int dim)

    int unur_urng_reset(UNUR_URNG* urng)

    int unur_urng_sync(UNUR_URNG* urng)

    int unur_urng_seed(UNUR_URNG* urng, unsigned long seed)

    int unur_urng_anti(UNUR_URNG* urng, int anti)

    int unur_urng_nextsub(UNUR_URNG* urng)

    int unur_urng_resetsub(UNUR_URNG* urng)

    int unur_gen_sync(UNUR_GEN* generator)

    int unur_gen_seed(UNUR_GEN* generator, unsigned long seed)

    int unur_gen_anti(UNUR_GEN* generator, int anti)

    int unur_gen_reset(UNUR_GEN* generator)

    int unur_gen_nextsub(UNUR_GEN* generator)

    int unur_gen_resetsub(UNUR_GEN* generator)

    ctypedef double (*_unur_urng_new_sampleunif_ft)(void* state)

    UNUR_URNG* unur_urng_new(_unur_urng_new_sampleunif_ft sampleunif, void* state)

    void unur_urng_free(UNUR_URNG* urng)

    ctypedef unsigned int (*_unur_urng_set_sample_array_samplearray_ft)(void* state, double* X, int dim)

    int unur_urng_set_sample_array(UNUR_URNG* urng, _unur_urng_set_sample_array_samplearray_ft samplearray)

    ctypedef void (*_unur_urng_set_sync_sync_ft)(void* state)

    int unur_urng_set_sync(UNUR_URNG* urng, _unur_urng_set_sync_sync_ft sync)

    ctypedef void (*_unur_urng_set_seed_setseed_ft)(void* state, unsigned long seed)

    int unur_urng_set_seed(UNUR_URNG* urng, _unur_urng_set_seed_setseed_ft setseed)

    ctypedef void (*_unur_urng_set_anti_setanti_ft)(void* state, int anti)

    int unur_urng_set_anti(UNUR_URNG* urng, _unur_urng_set_anti_setanti_ft setanti)

    ctypedef void (*_unur_urng_set_reset_reset_ft)(void* state)

    int unur_urng_set_reset(UNUR_URNG* urng, _unur_urng_set_reset_reset_ft reset)

    ctypedef void (*_unur_urng_set_nextsub_nextsub_ft)(void* state)

    int unur_urng_set_nextsub(UNUR_URNG* urng, _unur_urng_set_nextsub_nextsub_ft nextsub)

    ctypedef void (*_unur_urng_set_resetsub_resetsub_ft)(void* state)

    int unur_urng_set_resetsub(UNUR_URNG* urng, _unur_urng_set_resetsub_resetsub_ft resetsub)

    ctypedef void (*_unur_urng_set_delete_fpdelete_ft)(void* state)

    int unur_urng_set_delete(UNUR_URNG* urng, _unur_urng_set_delete_fpdelete_ft fpdelete)

    cdef enum:
        UNUR_DISTR_CONT
        UNUR_DISTR_CEMP
        UNUR_DISTR_CVEC
        UNUR_DISTR_CVEMP
        UNUR_DISTR_MATR
        UNUR_DISTR_DISCR

    void unur_distr_free(UNUR_DISTR* distribution)

    int unur_distr_set_name(UNUR_DISTR* distribution, char* name)

    char* unur_distr_get_name(UNUR_DISTR* distribution)

    int unur_distr_get_dim(UNUR_DISTR* distribution)

    unsigned int unur_distr_get_type(UNUR_DISTR* distribution)

    int unur_distr_is_cont(UNUR_DISTR* distribution)

    int unur_distr_is_cvec(UNUR_DISTR* distribution)

    int unur_distr_is_cemp(UNUR_DISTR* distribution)

    int unur_distr_is_cvemp(UNUR_DISTR* distribution)

    int unur_distr_is_discr(UNUR_DISTR* distribution)

    int unur_distr_is_matr(UNUR_DISTR* distribution)

    int unur_distr_set_extobj(UNUR_DISTR* distribution, void* extobj)

    void* unur_distr_get_extobj(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_clone(UNUR_DISTR* distr)

    UNUR_DISTR* unur_distr_cemp_new()

    int unur_distr_cemp_set_data(UNUR_DISTR* distribution, double* sample, int n_sample)

    int unur_distr_cemp_read_data(UNUR_DISTR* distribution, char* filename)

    int unur_distr_cemp_get_data(UNUR_DISTR* distribution, double** sample)

    int unur_distr_cemp_set_hist(UNUR_DISTR* distribution, double* prob, int n_prob, double xmin, double xmax)

    int unur_distr_cemp_set_hist_prob(UNUR_DISTR* distribution, double* prob, int n_prob)

    int unur_distr_cemp_set_hist_domain(UNUR_DISTR* distribution, double xmin, double xmax)

    int unur_distr_cemp_set_hist_bins(UNUR_DISTR* distribution, double* bins, int n_bins)

    UNUR_DISTR* unur_distr_cont_new()

    int unur_distr_cont_set_pdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* pdf)

    int unur_distr_cont_set_dpdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* dpdf)

    int unur_distr_cont_set_cdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* cdf)

    int unur_distr_cont_set_invcdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* invcdf)

    UNUR_FUNCT_CONT* unur_distr_cont_get_pdf(UNUR_DISTR* distribution)

    UNUR_FUNCT_CONT* unur_distr_cont_get_dpdf(UNUR_DISTR* distribution)

    UNUR_FUNCT_CONT* unur_distr_cont_get_cdf(UNUR_DISTR* distribution)

    UNUR_FUNCT_CONT* unur_distr_cont_get_invcdf(UNUR_DISTR* distribution)

    double unur_distr_cont_eval_pdf(double x, UNUR_DISTR* distribution)

    double unur_distr_cont_eval_dpdf(double x, UNUR_DISTR* distribution)

    double unur_distr_cont_eval_cdf(double x, UNUR_DISTR* distribution)

    double unur_distr_cont_eval_invcdf(double u, UNUR_DISTR* distribution)

    int unur_distr_cont_set_logpdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* logpdf)

    int unur_distr_cont_set_dlogpdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* dlogpdf)

    int unur_distr_cont_set_logcdf(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* logcdf)

    UNUR_FUNCT_CONT* unur_distr_cont_get_logpdf(UNUR_DISTR* distribution)

    UNUR_FUNCT_CONT* unur_distr_cont_get_dlogpdf(UNUR_DISTR* distribution)

    UNUR_FUNCT_CONT* unur_distr_cont_get_logcdf(UNUR_DISTR* distribution)

    double unur_distr_cont_eval_logpdf(double x, UNUR_DISTR* distribution)

    double unur_distr_cont_eval_dlogpdf(double x, UNUR_DISTR* distribution)

    double unur_distr_cont_eval_logcdf(double x, UNUR_DISTR* distribution)

    int unur_distr_cont_set_pdfstr(UNUR_DISTR* distribution, char* pdfstr)

    int unur_distr_cont_set_cdfstr(UNUR_DISTR* distribution, char* cdfstr)

    char* unur_distr_cont_get_pdfstr(UNUR_DISTR* distribution)

    char* unur_distr_cont_get_dpdfstr(UNUR_DISTR* distribution)

    char* unur_distr_cont_get_cdfstr(UNUR_DISTR* distribution)

    int unur_distr_cont_set_pdfparams(UNUR_DISTR* distribution, double* params, int n_params)

    int unur_distr_cont_get_pdfparams(UNUR_DISTR* distribution, double** params)

    int unur_distr_cont_set_pdfparams_vec(UNUR_DISTR* distribution, int par, double* param_vec, int n_param_vec)

    int unur_distr_cont_get_pdfparams_vec(UNUR_DISTR* distribution, int par, double** param_vecs)

    int unur_distr_cont_set_logpdfstr(UNUR_DISTR* distribution, char* logpdfstr)

    char* unur_distr_cont_get_logpdfstr(UNUR_DISTR* distribution)

    char* unur_distr_cont_get_dlogpdfstr(UNUR_DISTR* distribution)

    int unur_distr_cont_set_logcdfstr(UNUR_DISTR* distribution, char* logcdfstr)

    char* unur_distr_cont_get_logcdfstr(UNUR_DISTR* distribution)

    int unur_distr_cont_set_domain(UNUR_DISTR* distribution, double left, double right)

    int unur_distr_cont_get_domain(UNUR_DISTR* distribution, double* left, double* right)

    int unur_distr_cont_get_truncated(UNUR_DISTR* distribution, double* left, double* right)

    int unur_distr_cont_set_hr(UNUR_DISTR* distribution, UNUR_FUNCT_CONT* hazard)

    UNUR_FUNCT_CONT* unur_distr_cont_get_hr(UNUR_DISTR* distribution)

    double unur_distr_cont_eval_hr(double x, UNUR_DISTR* distribution)

    int unur_distr_cont_set_hrstr(UNUR_DISTR* distribution, char* hrstr)

    char* unur_distr_cont_get_hrstr(UNUR_DISTR* distribution)

    int unur_distr_cont_set_mode(UNUR_DISTR* distribution, double mode)

    int unur_distr_cont_upd_mode(UNUR_DISTR* distribution)

    double unur_distr_cont_get_mode(UNUR_DISTR* distribution)

    int unur_distr_cont_set_center(UNUR_DISTR* distribution, double center)

    double unur_distr_cont_get_center(UNUR_DISTR* distribution)

    int unur_distr_cont_set_pdfarea(UNUR_DISTR* distribution, double area)

    int unur_distr_cont_upd_pdfarea(UNUR_DISTR* distribution)

    double unur_distr_cont_get_pdfarea(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_cxtrans_new(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_cxtrans_get_distribution(UNUR_DISTR* distribution)

    int unur_distr_cxtrans_set_alpha(UNUR_DISTR* distribution, double alpha)

    int unur_distr_cxtrans_set_rescale(UNUR_DISTR* distribution, double mu, double sigma)

    double unur_distr_cxtrans_get_alpha(UNUR_DISTR* distribution)

    double unur_distr_cxtrans_get_mu(UNUR_DISTR* distribution)

    double unur_distr_cxtrans_get_sigma(UNUR_DISTR* distribution)

    int unur_distr_cxtrans_set_logpdfpole(UNUR_DISTR* distribution, double logpdfpole, double dlogpdfpole)

    int unur_distr_cxtrans_set_domain(UNUR_DISTR* distribution, double left, double right)

    UNUR_DISTR* unur_distr_corder_new(UNUR_DISTR* distribution, int n, int k)

    UNUR_DISTR* unur_distr_corder_get_distribution(UNUR_DISTR* distribution)

    int unur_distr_corder_set_rank(UNUR_DISTR* distribution, int n, int k)

    int unur_distr_corder_get_rank(UNUR_DISTR* distribution, int* n, int* k)

    UNUR_DISTR* unur_distr_cvec_new(int dim)

    int unur_distr_cvec_set_pdf(UNUR_DISTR* distribution, UNUR_FUNCT_CVEC* pdf)

    int unur_distr_cvec_set_dpdf(UNUR_DISTR* distribution, UNUR_VFUNCT_CVEC* dpdf)

    int unur_distr_cvec_set_pdpdf(UNUR_DISTR* distribution, UNUR_FUNCTD_CVEC* pdpdf)

    UNUR_FUNCT_CVEC* unur_distr_cvec_get_pdf(UNUR_DISTR* distribution)

    UNUR_VFUNCT_CVEC* unur_distr_cvec_get_dpdf(UNUR_DISTR* distribution)

    UNUR_FUNCTD_CVEC* unur_distr_cvec_get_pdpdf(UNUR_DISTR* distribution)

    double unur_distr_cvec_eval_pdf(double* x, UNUR_DISTR* distribution)

    int unur_distr_cvec_eval_dpdf(double* result, double* x, UNUR_DISTR* distribution)

    double unur_distr_cvec_eval_pdpdf(double* x, int coord, UNUR_DISTR* distribution)

    int unur_distr_cvec_set_logpdf(UNUR_DISTR* distribution, UNUR_FUNCT_CVEC* logpdf)

    int unur_distr_cvec_set_dlogpdf(UNUR_DISTR* distribution, UNUR_VFUNCT_CVEC* dlogpdf)

    int unur_distr_cvec_set_pdlogpdf(UNUR_DISTR* distribution, UNUR_FUNCTD_CVEC* pdlogpdf)

    UNUR_FUNCT_CVEC* unur_distr_cvec_get_logpdf(UNUR_DISTR* distribution)

    UNUR_VFUNCT_CVEC* unur_distr_cvec_get_dlogpdf(UNUR_DISTR* distribution)

    UNUR_FUNCTD_CVEC* unur_distr_cvec_get_pdlogpdf(UNUR_DISTR* distribution)

    double unur_distr_cvec_eval_logpdf(double* x, UNUR_DISTR* distribution)

    int unur_distr_cvec_eval_dlogpdf(double* result, double* x, UNUR_DISTR* distribution)

    double unur_distr_cvec_eval_pdlogpdf(double* x, int coord, UNUR_DISTR* distribution)

    int unur_distr_cvec_set_mean(UNUR_DISTR* distribution, double* mean)

    double* unur_distr_cvec_get_mean(UNUR_DISTR* distribution)

    int unur_distr_cvec_set_covar(UNUR_DISTR* distribution, double* covar)

    int unur_distr_cvec_set_covar_inv(UNUR_DISTR* distribution, double* covar_inv)

    double* unur_distr_cvec_get_covar(UNUR_DISTR* distribution)

    double* unur_distr_cvec_get_cholesky(UNUR_DISTR* distribution)

    double* unur_distr_cvec_get_covar_inv(UNUR_DISTR* distribution)

    int unur_distr_cvec_set_rankcorr(UNUR_DISTR* distribution, double* rankcorr)

    double* unur_distr_cvec_get_rankcorr(UNUR_DISTR* distribution)

    double* unur_distr_cvec_get_rk_cholesky(UNUR_DISTR* distribution)

    int unur_distr_cvec_set_marginals(UNUR_DISTR* distribution, UNUR_DISTR* marginal)

    int unur_distr_cvec_set_marginal_array(UNUR_DISTR* distribution, UNUR_DISTR** marginals)

    int unur_distr_cvec_set_marginal_list(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_cvec_get_marginal(UNUR_DISTR* distribution, int n)

    int unur_distr_cvec_set_pdfparams(UNUR_DISTR* distribution, double* params, int n_params)

    int unur_distr_cvec_get_pdfparams(UNUR_DISTR* distribution, double** params)

    int unur_distr_cvec_set_pdfparams_vec(UNUR_DISTR* distribution, int par, double* param_vec, int n_params)

    int unur_distr_cvec_get_pdfparams_vec(UNUR_DISTR* distribution, int par, double** param_vecs)

    int unur_distr_cvec_set_domain_rect(UNUR_DISTR* distribution, double* lowerleft, double* upperright)

    int unur_distr_cvec_is_indomain(double* x, UNUR_DISTR* distribution)

    int unur_distr_cvec_set_mode(UNUR_DISTR* distribution, double* mode)

    int unur_distr_cvec_upd_mode(UNUR_DISTR* distribution)

    double* unur_distr_cvec_get_mode(UNUR_DISTR* distribution)

    int unur_distr_cvec_set_center(UNUR_DISTR* distribution, double* center)

    double* unur_distr_cvec_get_center(UNUR_DISTR* distribution)

    int unur_distr_cvec_set_pdfvol(UNUR_DISTR* distribution, double volume)

    int unur_distr_cvec_upd_pdfvol(UNUR_DISTR* distribution)

    double unur_distr_cvec_get_pdfvol(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_condi_new(UNUR_DISTR* distribution, double* pos, double* dir, int k)

    int unur_distr_condi_set_condition(unur_distr* distribution, double* pos, double* dir, int k)

    int unur_distr_condi_get_condition(unur_distr* distribution, double** pos, double** dir, int* k)

    UNUR_DISTR* unur_distr_condi_get_distribution(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_cvemp_new(int dim)

    int unur_distr_cvemp_set_data(UNUR_DISTR* distribution, double* sample, int n_sample)

    int unur_distr_cvemp_read_data(UNUR_DISTR* distribution, char* filename)

    int unur_distr_cvemp_get_data(UNUR_DISTR* distribution, double** sample)

    UNUR_DISTR* unur_distr_discr_new()

    int unur_distr_discr_set_pv(UNUR_DISTR* distribution, double* pv, int n_pv)

    int unur_distr_discr_make_pv(UNUR_DISTR* distribution)

    int unur_distr_discr_get_pv(UNUR_DISTR* distribution, double** pv)

    int unur_distr_discr_set_pmf(UNUR_DISTR* distribution, UNUR_FUNCT_DISCR* pmf)

    int unur_distr_discr_set_cdf(UNUR_DISTR* distribution, UNUR_FUNCT_DISCR* cdf)

    int unur_distr_discr_set_invcdf(UNUR_DISTR* distribution, UNUR_IFUNCT_DISCR* invcdf)

    UNUR_FUNCT_DISCR* unur_distr_discr_get_pmf(UNUR_DISTR* distribution)

    UNUR_FUNCT_DISCR* unur_distr_discr_get_cdf(UNUR_DISTR* distribution)

    UNUR_IFUNCT_DISCR* unur_distr_discr_get_invcdf(UNUR_DISTR* distribution)

    double unur_distr_discr_eval_pv(int k, UNUR_DISTR* distribution)

    double unur_distr_discr_eval_pmf(int k, UNUR_DISTR* distribution)

    double unur_distr_discr_eval_cdf(int k, UNUR_DISTR* distribution)

    int unur_distr_discr_eval_invcdf(double u, UNUR_DISTR* distribution)

    int unur_distr_discr_set_pmfstr(UNUR_DISTR* distribution, char* pmfstr)

    int unur_distr_discr_set_cdfstr(UNUR_DISTR* distribution, char* cdfstr)

    char* unur_distr_discr_get_pmfstr(UNUR_DISTR* distribution)

    char* unur_distr_discr_get_cdfstr(UNUR_DISTR* distribution)

    int unur_distr_discr_set_pmfparams(UNUR_DISTR* distribution, double* params, int n_params)

    int unur_distr_discr_get_pmfparams(UNUR_DISTR* distribution, double** params)

    int unur_distr_discr_set_domain(UNUR_DISTR* distribution, int left, int right)

    int unur_distr_discr_get_domain(UNUR_DISTR* distribution, int* left, int* right)

    int unur_distr_discr_set_mode(UNUR_DISTR* distribution, int mode)

    int unur_distr_discr_upd_mode(UNUR_DISTR* distribution)

    int unur_distr_discr_get_mode(UNUR_DISTR* distribution)

    int unur_distr_discr_set_pmfsum(UNUR_DISTR* distribution, double sum)

    int unur_distr_discr_upd_pmfsum(UNUR_DISTR* distribution)

    double unur_distr_discr_get_pmfsum(UNUR_DISTR* distribution)

    UNUR_DISTR* unur_distr_matr_new(int n_rows, int n_cols)

    int unur_distr_matr_get_dim(UNUR_DISTR* distribution, int* n_rows, int* n_cols)

    UNUR_PAR* unur_auto_new(UNUR_DISTR* distribution)

    int unur_auto_set_logss(UNUR_PAR* parameters, int logss)

    UNUR_PAR* unur_dari_new(UNUR_DISTR* distribution)

    int unur_dari_set_squeeze(UNUR_PAR* parameters, int squeeze)

    int unur_dari_set_tablesize(UNUR_PAR* parameters, int size)

    int unur_dari_set_cpfactor(UNUR_PAR* parameters, double cp_factor)

    int unur_dari_set_verify(UNUR_PAR* parameters, int verify)

    int unur_dari_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_dau_new(UNUR_DISTR* distribution)

    int unur_dau_set_urnfactor(UNUR_PAR* parameters, double factor)

    UNUR_PAR* unur_dgt_new(UNUR_DISTR* distribution)

    int unur_dgt_set_guidefactor(UNUR_PAR* parameters, double factor)

    int unur_dgt_set_variant(UNUR_PAR* parameters, unsigned variant)

    int unur_dgt_eval_invcdf_recycle(UNUR_GEN* generator, double u, double* recycle)

    int unur_dgt_eval_invcdf(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_dsrou_new(UNUR_DISTR* distribution)

    int unur_dsrou_set_cdfatmode(UNUR_PAR* parameters, double Fmode)

    int unur_dsrou_set_verify(UNUR_PAR* parameters, int verify)

    int unur_dsrou_chg_verify(UNUR_GEN* generator, int verify)

    int unur_dsrou_chg_cdfatmode(UNUR_GEN* generator, double Fmode)

    UNUR_PAR* unur_dss_new(UNUR_DISTR* distribution)

    UNUR_PAR* unur_arou_new(UNUR_DISTR* distribution)

    int unur_arou_set_usedars(UNUR_PAR* parameters, int usedars)

    int unur_arou_set_darsfactor(UNUR_PAR* parameters, double factor)

    int unur_arou_set_max_sqhratio(UNUR_PAR* parameters, double max_ratio)

    double unur_arou_get_sqhratio(UNUR_GEN* generator)

    double unur_arou_get_hatarea(UNUR_GEN* generator)

    double unur_arou_get_squeezearea(UNUR_GEN* generator)

    int unur_arou_set_max_segments(UNUR_PAR* parameters, int max_segs)

    int unur_arou_set_cpoints(UNUR_PAR* parameters, int n_stp, double* stp)

    int unur_arou_set_usecenter(UNUR_PAR* parameters, int usecenter)

    int unur_arou_set_guidefactor(UNUR_PAR* parameters, double factor)

    int unur_arou_set_verify(UNUR_PAR* parameters, int verify)

    int unur_arou_chg_verify(UNUR_GEN* generator, int verify)

    int unur_arou_set_pedantic(UNUR_PAR* parameters, int pedantic)

    UNUR_PAR* unur_ars_new(UNUR_DISTR* distribution)

    int unur_ars_set_max_intervals(UNUR_PAR* parameters, int max_ivs)

    int unur_ars_set_cpoints(UNUR_PAR* parameters, int n_cpoints, double* cpoints)

    int unur_ars_set_reinit_percentiles(UNUR_PAR* parameters, int n_percentiles, double* percentiles)

    int unur_ars_chg_reinit_percentiles(UNUR_GEN* generator, int n_percentiles, double* percentiles)

    int unur_ars_set_reinit_ncpoints(UNUR_PAR* parameters, int ncpoints)

    int unur_ars_chg_reinit_ncpoints(UNUR_GEN* generator, int ncpoints)

    int unur_ars_set_max_iter(UNUR_PAR* parameters, int max_iter)

    int unur_ars_set_verify(UNUR_PAR* parameters, int verify)

    int unur_ars_chg_verify(UNUR_GEN* generator, int verify)

    int unur_ars_set_pedantic(UNUR_PAR* parameters, int pedantic)

    double unur_ars_get_loghatarea(UNUR_GEN* generator)

    double unur_ars_eval_invcdfhat(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_hinv_new(UNUR_DISTR* distribution)

    int unur_hinv_set_order(UNUR_PAR* parameters, int order)

    int unur_hinv_set_u_resolution(UNUR_PAR* parameters, double u_resolution)

    int unur_hinv_set_cpoints(UNUR_PAR* parameters, double* stp, int n_stp)

    int unur_hinv_set_boundary(UNUR_PAR* parameters, double left, double right)

    int unur_hinv_set_guidefactor(UNUR_PAR* parameters, double factor)

    int unur_hinv_set_max_intervals(UNUR_PAR* parameters, int max_ivs)

    int unur_hinv_get_n_intervals(UNUR_GEN* generator)

    double unur_hinv_eval_approxinvcdf(UNUR_GEN* generator, double u)

    int unur_hinv_chg_truncated(UNUR_GEN* generator, double left, double right)

    int unur_hinv_estimate_error(UNUR_GEN* generator, int samplesize, double* max_error, double* MAE)

    UNUR_PAR* unur_hrb_new(UNUR_DISTR* distribution)

    int unur_hrb_set_upperbound(UNUR_PAR* parameters, double upperbound)

    int unur_hrb_set_verify(UNUR_PAR* parameters, int verify)

    int unur_hrb_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_hrd_new(UNUR_DISTR* distribution)

    int unur_hrd_set_verify(UNUR_PAR* parameters, int verify)

    int unur_hrd_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_hri_new(UNUR_DISTR* distribution)

    int unur_hri_set_p0(UNUR_PAR* parameters, double p0)

    int unur_hri_set_verify(UNUR_PAR* parameters, int verify)

    int unur_hri_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_itdr_new(UNUR_DISTR* distribution)

    int unur_itdr_set_xi(UNUR_PAR* parameters, double xi)

    int unur_itdr_set_cp(UNUR_PAR* parameters, double cp)

    int unur_itdr_set_ct(UNUR_PAR* parameters, double ct)

    double unur_itdr_get_xi(UNUR_GEN* generator)

    double unur_itdr_get_cp(UNUR_GEN* generator)

    double unur_itdr_get_ct(UNUR_GEN* generator)

    double unur_itdr_get_area(UNUR_GEN* generator)

    int unur_itdr_set_verify(UNUR_PAR* parameters, int verify)

    int unur_itdr_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_mcorr_new(UNUR_DISTR* distribution)

    int unur_mcorr_set_eigenvalues(UNUR_PAR* par, double* eigenvalues)

    int unur_mcorr_chg_eigenvalues(UNUR_GEN* gen, double* eigenvalues)

    UNUR_PAR* unur_ninv_new(UNUR_DISTR* distribution)

    int unur_ninv_set_useregula(UNUR_PAR* parameters)

    int unur_ninv_set_usenewton(UNUR_PAR* parameters)

    int unur_ninv_set_usebisect(UNUR_PAR* parameters)

    int unur_ninv_set_max_iter(UNUR_PAR* parameters, int max_iter)

    int unur_ninv_chg_max_iter(UNUR_GEN* generator, int max_iter)

    int unur_ninv_set_x_resolution(UNUR_PAR* parameters, double x_resolution)

    int unur_ninv_chg_x_resolution(UNUR_GEN* generator, double x_resolution)

    int unur_ninv_set_u_resolution(UNUR_PAR* parameters, double u_resolution)

    int unur_ninv_chg_u_resolution(UNUR_GEN* generator, double u_resolution)

    int unur_ninv_set_start(UNUR_PAR* parameters, double left, double right)

    int unur_ninv_chg_start(UNUR_GEN* gen, double left, double right)

    int unur_ninv_set_table(UNUR_PAR* parameters, int no_of_points)

    int unur_ninv_chg_table(UNUR_GEN* gen, int no_of_points)

    int unur_ninv_chg_truncated(UNUR_GEN* gen, double left, double right)

    double unur_ninv_eval_approxinvcdf(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_nrou_new(UNUR_DISTR* distribution)

    int unur_nrou_set_u(UNUR_PAR* parameters, double umin, double umax)

    int unur_nrou_set_v(UNUR_PAR* parameters, double vmax)

    int unur_nrou_set_r(UNUR_PAR* parameters, double r)

    int unur_nrou_set_center(UNUR_PAR* parameters, double center)

    int unur_nrou_set_verify(UNUR_PAR* parameters, int verify)

    int unur_nrou_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_pinv_new(UNUR_DISTR* distribution)

    int unur_pinv_set_order(UNUR_PAR* parameters, int order)

    int unur_pinv_set_smoothness(UNUR_PAR* parameters, int smoothness)

    int unur_pinv_set_u_resolution(UNUR_PAR* parameters, double u_resolution)

    int unur_pinv_set_use_upoints(UNUR_PAR* parameters, int use_upoints)

    int unur_pinv_set_usepdf(UNUR_PAR* parameters)

    int unur_pinv_set_usecdf(UNUR_PAR* parameters)

    int unur_pinv_set_boundary(UNUR_PAR* parameters, double left, double right)

    int unur_pinv_set_searchboundary(UNUR_PAR* parameters, int left, int right)

    int unur_pinv_set_max_intervals(UNUR_PAR* parameters, int max_ivs)

    int unur_pinv_get_n_intervals(UNUR_GEN* generator)

    int unur_pinv_set_keepcdf(UNUR_PAR* parameters, int keepcdf)

    double unur_pinv_eval_approxinvcdf(UNUR_GEN* generator, double u)

    double unur_pinv_eval_approxcdf(UNUR_GEN* generator, double x)

    int unur_pinv_estimate_error(UNUR_GEN* generator, int samplesize, double* max_error, double* MAE)

    UNUR_PAR* unur_srou_new(UNUR_DISTR* distribution)

    int unur_srou_set_r(UNUR_PAR* parameters, double r)

    int unur_srou_set_cdfatmode(UNUR_PAR* parameters, double Fmode)

    int unur_srou_set_pdfatmode(UNUR_PAR* parameters, double fmode)

    int unur_srou_set_usesqueeze(UNUR_PAR* parameters, int usesqueeze)

    int unur_srou_set_usemirror(UNUR_PAR* parameters, int usemirror)

    int unur_srou_set_verify(UNUR_PAR* parameters, int verify)

    int unur_srou_chg_verify(UNUR_GEN* generator, int verify)

    int unur_srou_chg_cdfatmode(UNUR_GEN* generator, double Fmode)

    int unur_srou_chg_pdfatmode(UNUR_GEN* generator, double fmode)

    UNUR_PAR* unur_ssr_new(UNUR_DISTR* distribution)

    int unur_ssr_set_cdfatmode(UNUR_PAR* parameters, double Fmode)

    int unur_ssr_set_pdfatmode(UNUR_PAR* parameters, double fmode)

    int unur_ssr_set_usesqueeze(UNUR_PAR* parameters, int usesqueeze)

    int unur_ssr_set_verify(UNUR_PAR* parameters, int verify)

    int unur_ssr_chg_verify(UNUR_GEN* generator, int verify)

    int unur_ssr_chg_cdfatmode(UNUR_GEN* generator, double Fmode)

    int unur_ssr_chg_pdfatmode(UNUR_GEN* generator, double fmode)

    UNUR_PAR* unur_tabl_new(UNUR_DISTR* distribution)

    int unur_tabl_set_variant_ia(UNUR_PAR* parameters, int use_ia)

    int unur_tabl_set_cpoints(UNUR_PAR* parameters, int n_cpoints, double* cpoints)

    int unur_tabl_set_nstp(UNUR_PAR* parameters, int n_stp)

    int unur_tabl_set_useear(UNUR_PAR* parameters, int useear)

    int unur_tabl_set_areafraction(UNUR_PAR* parameters, double fraction)

    int unur_tabl_set_usedars(UNUR_PAR* parameters, int usedars)

    int unur_tabl_set_darsfactor(UNUR_PAR* parameters, double factor)

    int unur_tabl_set_variant_splitmode(UNUR_PAR* parameters, unsigned splitmode)

    int unur_tabl_set_max_sqhratio(UNUR_PAR* parameters, double max_ratio)

    double unur_tabl_get_sqhratio(UNUR_GEN* generator)

    double unur_tabl_get_hatarea(UNUR_GEN* generator)

    double unur_tabl_get_squeezearea(UNUR_GEN* generator)

    int unur_tabl_set_max_intervals(UNUR_PAR* parameters, int max_ivs)

    int unur_tabl_get_n_intervals(UNUR_GEN* generator)

    int unur_tabl_set_slopes(UNUR_PAR* parameters, double* slopes, int n_slopes)

    int unur_tabl_set_guidefactor(UNUR_PAR* parameters, double factor)

    int unur_tabl_set_boundary(UNUR_PAR* parameters, double left, double right)

    int unur_tabl_chg_truncated(UNUR_GEN* gen, double left, double right)

    int unur_tabl_set_verify(UNUR_PAR* parameters, int verify)

    int unur_tabl_chg_verify(UNUR_GEN* generator, int verify)

    int unur_tabl_set_pedantic(UNUR_PAR* parameters, int pedantic)

    UNUR_PAR* unur_tdr_new(UNUR_DISTR* distribution)

    int unur_tdr_set_c(UNUR_PAR* parameters, double c)

    int unur_tdr_set_variant_gw(UNUR_PAR* parameters)

    int unur_tdr_set_variant_ps(UNUR_PAR* parameters)

    int unur_tdr_set_variant_ia(UNUR_PAR* parameters)

    int unur_tdr_set_usedars(UNUR_PAR* parameters, int usedars)

    int unur_tdr_set_darsfactor(UNUR_PAR* parameters, double factor)

    int unur_tdr_set_cpoints(UNUR_PAR* parameters, int n_stp, double* stp)

    int unur_tdr_set_reinit_percentiles(UNUR_PAR* parameters, int n_percentiles, double* percentiles)

    int unur_tdr_chg_reinit_percentiles(UNUR_GEN* generator, int n_percentiles, double* percentiles)

    int unur_tdr_set_reinit_ncpoints(UNUR_PAR* parameters, int ncpoints)

    int unur_tdr_chg_reinit_ncpoints(UNUR_GEN* generator, int ncpoints)

    int unur_tdr_chg_truncated(UNUR_GEN* gen, double left, double right)

    int unur_tdr_set_max_sqhratio(UNUR_PAR* parameters, double max_ratio)

    double unur_tdr_get_sqhratio(UNUR_GEN* generator)

    double unur_tdr_get_hatarea(UNUR_GEN* generator)

    double unur_tdr_get_squeezearea(UNUR_GEN* generator)

    int unur_tdr_set_max_intervals(UNUR_PAR* parameters, int max_ivs)

    int unur_tdr_set_usecenter(UNUR_PAR* parameters, int usecenter)

    int unur_tdr_set_usemode(UNUR_PAR* parameters, int usemode)

    int unur_tdr_set_guidefactor(UNUR_PAR* parameters, double factor)

    int unur_tdr_set_verify(UNUR_PAR* parameters, int verify)

    int unur_tdr_chg_verify(UNUR_GEN* generator, int verify)

    int unur_tdr_set_pedantic(UNUR_PAR* parameters, int pedantic)

    double unur_tdr_eval_invcdfhat(UNUR_GEN* generator, double u, double* hx, double* fx, double* sqx)

    int _unur_tdr_is_ARS_running(UNUR_GEN* generator)

    UNUR_PAR* unur_utdr_new(UNUR_DISTR* distribution)

    int unur_utdr_set_pdfatmode(UNUR_PAR* parameters, double fmode)

    int unur_utdr_set_cpfactor(UNUR_PAR* parameters, double cp_factor)

    int unur_utdr_set_deltafactor(UNUR_PAR* parameters, double delta)

    int unur_utdr_set_verify(UNUR_PAR* parameters, int verify)

    int unur_utdr_chg_verify(UNUR_GEN* generator, int verify)

    int unur_utdr_chg_pdfatmode(UNUR_GEN* generator, double fmode)

    UNUR_PAR* unur_empk_new(UNUR_DISTR* distribution)

    int unur_empk_set_kernel(UNUR_PAR* parameters, unsigned kernel)

    int unur_empk_set_kernelgen(UNUR_PAR* parameters, UNUR_GEN* kernelgen, double alpha, double kernelvar)

    int unur_empk_set_beta(UNUR_PAR* parameters, double beta)

    int unur_empk_set_smoothing(UNUR_PAR* parameters, double smoothing)

    int unur_empk_chg_smoothing(UNUR_GEN* generator, double smoothing)

    int unur_empk_set_varcor(UNUR_PAR* parameters, int varcor)

    int unur_empk_chg_varcor(UNUR_GEN* generator, int varcor)

    int unur_empk_set_positive(UNUR_PAR* parameters, int positive)

    UNUR_PAR* unur_empl_new(UNUR_DISTR* distribution)

    UNUR_PAR* unur_hist_new(UNUR_DISTR* distribution)

    UNUR_PAR* unur_mvtdr_new(UNUR_DISTR* distribution)

    int unur_mvtdr_set_stepsmin(UNUR_PAR* parameters, int stepsmin)

    int unur_mvtdr_set_boundsplitting(UNUR_PAR* parameters, double boundsplitting)

    int unur_mvtdr_set_maxcones(UNUR_PAR* parameters, int maxcones)

    int unur_mvtdr_get_ncones(UNUR_GEN* generator)

    double unur_mvtdr_get_hatvol(UNUR_GEN* generator)

    int unur_mvtdr_set_verify(UNUR_PAR* parameters, int verify)

    int unur_mvtdr_chg_verify(UNUR_GEN* generator, int verify)

    UNUR_PAR* unur_norta_new(UNUR_DISTR* distribution)

    UNUR_PAR* unur_vempk_new(UNUR_DISTR* distribution)

    int unur_vempk_set_smoothing(UNUR_PAR* parameters, double smoothing)

    int unur_vempk_chg_smoothing(UNUR_GEN* generator, double smoothing)

    int unur_vempk_set_varcor(UNUR_PAR* parameters, int varcor)

    int unur_vempk_chg_varcor(UNUR_GEN* generator, int varcor)

    UNUR_PAR* unur_gibbs_new(UNUR_DISTR* distribution)

    int unur_gibbs_set_variant_coordinate(UNUR_PAR* parameters)

    int unur_gibbs_set_variant_random_direction(UNUR_PAR* parameters)

    int unur_gibbs_set_c(UNUR_PAR* parameters, double c)

    int unur_gibbs_set_startingpoint(UNUR_PAR* parameters, double* x0)

    int unur_gibbs_set_thinning(UNUR_PAR* parameters, int thinning)

    int unur_gibbs_set_burnin(UNUR_PAR* parameters, int burnin)

    double* unur_gibbs_get_state(UNUR_GEN* generator)

    int unur_gibbs_chg_state(UNUR_GEN* generator, double* state)

    int unur_gibbs_reset_state(UNUR_GEN* generator)

    UNUR_PAR* unur_cstd_new(UNUR_DISTR* distribution)

    int unur_cstd_set_variant(UNUR_PAR* parameters, unsigned variant)

    int unur_cstd_chg_truncated(UNUR_GEN* generator, double left, double right)

    double unur_cstd_eval_invcdf(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_dstd_new(UNUR_DISTR* distribution)

    int unur_dstd_set_variant(UNUR_PAR* parameters, unsigned variant)

    int unur_dstd_chg_truncated(UNUR_GEN* generator, int left, int right)

    int unur_dstd_eval_invcdf(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_mvstd_new(UNUR_DISTR* distribution)

    UNUR_PAR* unur_mixt_new(int n, double* prob, UNUR_GEN** comp)

    int unur_mixt_set_useinversion(UNUR_PAR* parameters, int useinv)

    double unur_mixt_eval_invcdf(UNUR_GEN* generator, double u)

    UNUR_PAR* unur_cext_new(UNUR_DISTR* distribution)

    ctypedef int (*_unur_cext_set_init_init_ft)(UNUR_GEN* gen)

    int unur_cext_set_init(UNUR_PAR* parameters, _unur_cext_set_init_init_ft init)

    ctypedef double (*_unur_cext_set_sample_sample_ft)(UNUR_GEN* gen)

    int unur_cext_set_sample(UNUR_PAR* parameters, _unur_cext_set_sample_sample_ft sample)

    void* unur_cext_get_params(UNUR_GEN* generator, size_t size)

    double* unur_cext_get_distrparams(UNUR_GEN* generator)

    int unur_cext_get_ndistrparams(UNUR_GEN* generator)

    UNUR_PAR* unur_dext_new(UNUR_DISTR* distribution)

    ctypedef int (*_unur_dext_set_init_init_ft)(UNUR_GEN* gen)

    int unur_dext_set_init(UNUR_PAR* parameters, _unur_dext_set_init_init_ft init)

    ctypedef int (*_unur_dext_set_sample_sample_ft)(UNUR_GEN* gen)

    int unur_dext_set_sample(UNUR_PAR* parameters, _unur_dext_set_sample_sample_ft sample)

    void* unur_dext_get_params(UNUR_GEN* generator, size_t size)

    double* unur_dext_get_distrparams(UNUR_GEN* generator)

    int unur_dext_get_ndistrparams(UNUR_GEN* generator)

    UNUR_PAR* unur_unif_new(UNUR_DISTR* dummy)

    UNUR_GEN* unur_str2gen(char* string)

    UNUR_DISTR* unur_str2distr(char* string)

    UNUR_GEN* unur_makegen_ssu(char* distrstr, char* methodstr, UNUR_URNG* urng)

    UNUR_GEN* unur_makegen_dsu(UNUR_DISTR* distribution, char* methodstr, UNUR_URNG* urng)

    UNUR_PAR* _unur_str2par(UNUR_DISTR* distribution, char* method, unur_slist** mlist)

    UNUR_GEN* unur_init(UNUR_PAR* parameters)

    int unur_reinit(UNUR_GEN* generator)

    int unur_sample_discr(UNUR_GEN* generator)

    double unur_sample_cont(UNUR_GEN* generator)

    int unur_sample_vec(UNUR_GEN* generator, double* vector)

    int unur_sample_matr(UNUR_GEN* generator, double* matrix)

    double unur_quantile(UNUR_GEN* generator, double U)

    void unur_free(UNUR_GEN* generator)

    char* unur_gen_info(UNUR_GEN* generator, int help)

    int unur_get_dimension(UNUR_GEN* generator)

    char* unur_get_genid(UNUR_GEN* generator)

    UNUR_DISTR* unur_get_distr(UNUR_GEN* generator)

    int unur_set_use_distr_privatecopy(UNUR_PAR* parameters, int use_privatecopy)

    UNUR_GEN* unur_gen_clone(UNUR_GEN* gen)

    void unur_par_free(UNUR_PAR* par)

    cdef enum:
        UNUR_DISTR_GENERIC
        UNUR_DISTR_CORDER
        UNUR_DISTR_CXTRANS
        UNUR_DISTR_CONDI
        UNUR_DISTR_BETA
        UNUR_DISTR_CAUCHY
        UNUR_DISTR_CHI
        UNUR_DISTR_CHISQUARE
        UNUR_DISTR_EPANECHNIKOV
        UNUR_DISTR_EXPONENTIAL
        UNUR_DISTR_EXTREME_I
        UNUR_DISTR_EXTREME_II
        UNUR_DISTR_F
        UNUR_DISTR_GAMMA
        UNUR_DISTR_GHYP
        UNUR_DISTR_GIG
        UNUR_DISTR_GIG2
        UNUR_DISTR_HYPERBOLIC
        UNUR_DISTR_IG
        UNUR_DISTR_LAPLACE
        UNUR_DISTR_LOGISTIC
        UNUR_DISTR_LOGNORMAL
        UNUR_DISTR_LOMAX
        UNUR_DISTR_NORMAL
        UNUR_DISTR_GAUSSIAN
        UNUR_DISTR_PARETO
        UNUR_DISTR_POWEREXPONENTIAL
        UNUR_DISTR_RAYLEIGH
        UNUR_DISTR_SLASH
        UNUR_DISTR_STUDENT
        UNUR_DISTR_TRIANGULAR
        UNUR_DISTR_UNIFORM
        UNUR_DISTR_BOXCAR
        UNUR_DISTR_WEIBULL
        UNUR_DISTR_BURR_I
        UNUR_DISTR_BURR_II
        UNUR_DISTR_BURR_III
        UNUR_DISTR_BURR_IV
        UNUR_DISTR_BURR_V
        UNUR_DISTR_BURR_VI
        UNUR_DISTR_BURR_VII
        UNUR_DISTR_BURR_VIII
        UNUR_DISTR_BURR_IX
        UNUR_DISTR_BURR_X
        UNUR_DISTR_BURR_XI
        UNUR_DISTR_BURR_XII
        UNUR_DISTR_BINOMIAL
        UNUR_DISTR_GEOMETRIC
        UNUR_DISTR_HYPERGEOMETRIC
        UNUR_DISTR_LOGARITHMIC
        UNUR_DISTR_NEGATIVEBINOMIAL
        UNUR_DISTR_POISSON
        UNUR_DISTR_ZIPF
        UNUR_DISTR_MCAUCHY
        UNUR_DISTR_MNORMAL
        UNUR_DISTR_MSTUDENT
        UNUR_DISTR_MEXPONENTIAL
        UNUR_DISTR_COPULA
        UNUR_DISTR_MCORRELATION

    UNUR_DISTR* unur_distr_beta(double* params, int n_params)

    UNUR_DISTR* unur_distr_burr(double* params, int n_params)

    UNUR_DISTR* unur_distr_cauchy(double* params, int n_params)

    UNUR_DISTR* unur_distr_chi(double* params, int n_params)

    UNUR_DISTR* unur_distr_chisquare(double* params, int n_params)

    UNUR_DISTR* unur_distr_exponential(double* params, int n_params)

    UNUR_DISTR* unur_distr_extremeI(double* params, int n_params)

    UNUR_DISTR* unur_distr_extremeII(double* params, int n_params)

    UNUR_DISTR* unur_distr_F(double* params, int n_params)

    UNUR_DISTR* unur_distr_gamma(double* params, int n_params)

    UNUR_DISTR* unur_distr_ghyp(double* params, int n_params)

    UNUR_DISTR* unur_distr_gig(double* params, int n_params)

    UNUR_DISTR* unur_distr_gig2(double* params, int n_params)

    UNUR_DISTR* unur_distr_hyperbolic(double* params, int n_params)

    UNUR_DISTR* unur_distr_ig(double* params, int n_params)

    UNUR_DISTR* unur_distr_laplace(double* params, int n_params)

    UNUR_DISTR* unur_distr_logistic(double* params, int n_params)

    UNUR_DISTR* unur_distr_lognormal(double* params, int n_params)

    UNUR_DISTR* unur_distr_lomax(double* params, int n_params)

    UNUR_DISTR* unur_distr_normal(double* params, int n_params)

    UNUR_DISTR* unur_distr_pareto(double* params, int n_params)

    UNUR_DISTR* unur_distr_powerexponential(double* params, int n_params)

    UNUR_DISTR* unur_distr_rayleigh(double* params, int n_params)

    UNUR_DISTR* unur_distr_slash(double* params, int n_params)

    UNUR_DISTR* unur_distr_student(double* params, int n_params)

    UNUR_DISTR* unur_distr_triangular(double* params, int n_params)

    UNUR_DISTR* unur_distr_uniform(double* params, int n_params)

    UNUR_DISTR* unur_distr_weibull(double* params, int n_params)

    UNUR_DISTR* unur_distr_multinormal(int dim, double* mean, double* covar)

    UNUR_DISTR* unur_distr_multicauchy(int dim, double* mean, double* covar)

    UNUR_DISTR* unur_distr_multistudent(int dim, double nu, double* mean, double* covar)

    UNUR_DISTR* unur_distr_multiexponential(int dim, double* sigma, double* theta)

    UNUR_DISTR* unur_distr_copula(int dim, double* rankcorr)

    UNUR_DISTR* unur_distr_correlation(int n)

    UNUR_DISTR* unur_distr_binomial(double* params, int n_params)

    UNUR_DISTR* unur_distr_geometric(double* params, int n_params)

    UNUR_DISTR* unur_distr_hypergeometric(double* params, int n_params)

    UNUR_DISTR* unur_distr_logarithmic(double* params, int n_params)

    UNUR_DISTR* unur_distr_negativebinomial(double* params, int n_params)

    UNUR_DISTR* unur_distr_poisson(double* params, int n_params)

    UNUR_DISTR* unur_distr_zipf(double* params, int n_params)

    FILE* unur_set_stream(FILE* new_stream)

    FILE* unur_get_stream()

    int unur_set_debug(UNUR_PAR* parameters, unsigned debug)

    int unur_chg_debug(UNUR_GEN* generator, unsigned debug)

    int unur_set_default_debug(unsigned debug)

    int unur_errno

    int unur_get_errno()

    void unur_reset_errno()

    char* unur_get_strerror(int errnocode)

    UNUR_ERROR_HANDLER* unur_set_error_handler(UNUR_ERROR_HANDLER* new_handler)

    UNUR_ERROR_HANDLER* unur_set_error_handler_off()

    cdef enum:
        UNUR_SUCCESS
        UNUR_FAILURE
        UNUR_ERR_DISTR_SET
        UNUR_ERR_DISTR_GET
        UNUR_ERR_DISTR_NPARAMS
        UNUR_ERR_DISTR_DOMAIN
        UNUR_ERR_DISTR_GEN
        UNUR_ERR_DISTR_REQUIRED
        UNUR_ERR_DISTR_UNKNOWN
        UNUR_ERR_DISTR_INVALID
        UNUR_ERR_DISTR_DATA
        UNUR_ERR_DISTR_PROP
        UNUR_ERR_PAR_SET
        UNUR_ERR_PAR_VARIANT
        UNUR_ERR_PAR_INVALID
        UNUR_ERR_GEN
        UNUR_ERR_GEN_DATA
        UNUR_ERR_GEN_CONDITION
        UNUR_ERR_GEN_INVALID
        UNUR_ERR_GEN_SAMPLING
        UNUR_ERR_NO_REINIT
        UNUR_ERR_NO_QUANTILE
        UNUR_ERR_URNG
        UNUR_ERR_URNG_MISS
        UNUR_ERR_STR
        UNUR_ERR_STR_UNKNOWN
        UNUR_ERR_STR_SYNTAX
        UNUR_ERR_STR_INVALID
        UNUR_ERR_FSTR_SYNTAX
        UNUR_ERR_FSTR_DERIV
        UNUR_ERR_DOMAIN
        UNUR_ERR_ROUNDOFF
        UNUR_ERR_MALLOC
        UNUR_ERR_NULL
        UNUR_ERR_COOKIE
        UNUR_ERR_GENERIC
        UNUR_ERR_SILENT
        UNUR_ERR_INF
        UNUR_ERR_NAN
        UNUR_ERR_COMPILE
        UNUR_ERR_SHOULD_NOT_HAPPEN

    double INFINITY

    unur_slist* _unur_slist_new()

    int _unur_slist_append(unur_slist* slist, void* element)

    int _unur_slist_length(unur_slist* slist)

    void* _unur_slist_get(unur_slist* slist, int n)

    void* _unur_slist_replace(unur_slist* slist, int n, void* element)

    void _unur_slist_free(unur_slist* slist)



# This file is not meant for public use and will be removed in SciPy v2.0.0.
# Use the `scipy.signal` namespace for importing the functions
# included below.

from scipy._lib.deprecation import _sub_module_deprecation

__all__ = [  # noqa: F822
    'sawtooth', 'square', 'gausspulse', 'chirp', 'sweep_poly',
    'unit_impulse',
]


def __dir__():
    return __all__


def __getattr__(name):
    return _sub_module_deprecation(sub_package="signal", module="waveforms",
                                   private_modules=["_waveforms"], all=__all__,
                                   attribute=name)



# Temporary file separated from _distribution_infrastructure.py
# to simplify the diff during PR review.
from abc import ABC, abstractmethod

class _ProbabilityDistribution(ABC):
    @abstractmethod
    def support(self):
        r"""Support of the random variable

        The support of a random variable is set of all possible outcomes;
        i.e., the subset of the domain of argument :math:`x` for which
        the probability density function :math:`f(x)` is nonzero.

        This function returns lower and upper bounds of the support.

        Returns
        -------
        out : tuple of Array
            The lower and upper bounds of the support.

        See Also
        --------
        pdf

        References
        ----------
        .. [1] Support (mathematics), *Wikipedia*,
               https://en.wikipedia.org/wiki/Support_(mathematics)

        Notes
        -----
        Suppose a continuous probability distribution has support ``(l, r)``.
        The following table summarizes the value returned by methods
        of ``ContinuousDistribution`` for arguments outside the support.

        +----------------+---------------------+---------------------+
        | Method         | Value for ``x < l`` | Value for ``x > r`` |
        +================+=====================+=====================+
        | ``pdf(x)``     | 0                   | 0                   |
        +----------------+---------------------+---------------------+
        | ``logpdf(x)``  | -inf                | -inf                |
        +----------------+---------------------+---------------------+
        | ``cdf(x)``     | 0                   | 1                   |
        +----------------+---------------------+---------------------+
        | ``logcdf(x)``  | -inf                | 0                   |
        +----------------+---------------------+---------------------+
        | ``ccdf(x)``    | 1                   | 0                   |
        +----------------+---------------------+---------------------+
        | ``logccdf(x)`` | 0                   | -inf                |
        +----------------+---------------------+---------------------+

        For the ``cdf`` and related methods, the inequality need not be
        strict; i.e. the tabulated value is returned when the method is
        evaluated *at* the corresponding boundary.

        The following table summarizes the value returned by the inverse
        methods of ``ContinuousDistribution`` for arguments at the boundaries
        of the domain ``0`` to ``1``.

        +-------------+-----------+-----------+
        | Method      | ``x = 0`` | ``x = 1`` |
        +=============+===========+===========+
        | ``icdf(x)`` | ``l``     | ``r``     |
        +-------------+-----------+-----------+
        | ``icdf(x)`` | ``r``     | ``l``     |
        +-------------+-----------+-----------+

        For the inverse log-functions, the same values are returned for
        for ``x = log(0)`` and ``x = log(1)``. All inverse functions return
        ``nan`` when evaluated at an argument outside the domain ``0`` to ``1``.

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Retrieve the support of the distribution:

        >>> X.support()
        (-0.5, 0.5)

        For a distribution with infinite support,

        >>> X = stats.Normal()
        >>> X.support()
        (-inf, inf)

        Due to underflow, the numerical value returned by the PDF may be zero
        even for arguments within the support, even if the true value is
        nonzero. In such cases, the log-PDF may be useful.

        >>> X.pdf([-100., 100.])
        array([0., 0.])
        >>> X.logpdf([-100., 100.])
        array([-5000.91893853, -5000.91893853])

        Use cases for the log-CDF and related methods are analogous.

        """
        raise NotImplementedError()

    @abstractmethod
    def sample(self, shape, *, method, rng):
        r"""Random sample from the distribution.

        Parameters
        ----------
        shape : tuple of ints, default: ()
            The shape of the sample to draw. If the parameters of the distribution
            underlying the random variable are arrays of shape ``param_shape``,
            the output array will be of shape ``shape + param_shape``.
        method : {None, 'formula', 'inverse_transform'}
            The strategy used to produce the sample. By default (``None``),
            the infrastructure chooses between the following options,
            listed in order of precedence.

            - ``'formula'``: an implementation specific to the distribution
            - ``'inverse_transform'``: generate a uniformly distributed sample and
              return the inverse CDF at these arguments.

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a `NotImplementedError``
            will be raised.
        rng : `numpy.random.Generator` or `scipy.stats.QMCEngine`, optional
            Pseudo- or quasi-random number generator state. When `rng` is None,
            a new `numpy.random.Generator` is created using entropy from the
            operating system. Types other than `numpy.random.Generator` and
            `scipy.stats.QMCEngine` are passed to `numpy.random.default_rng`
            to instantiate a ``Generator``.

            If `rng` is an instance of `scipy.stats.QMCEngine` configured to use
            scrambling and `shape` is not empty, then each slice along the zeroth
            axis of the result is a "quasi-independent", low-discrepancy sequence;
            that is, they are distinct sequences that can be treated as statistically
            independent for most practical purposes. Separate calls to `sample`
            produce new quasi-independent, low-discrepancy sequences.

        References
        ----------
        .. [1] Sampling (statistics), *Wikipedia*,
               https://en.wikipedia.org/wiki/Sampling_(statistics)

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=0., b=1.)

        Generate a pseudorandom sample:

        >>> x = X.sample((1000, 1))
        >>> octiles = (np.arange(8) + 1) / 8
        >>> np.count_nonzero(x <= octiles, axis=0)
        array([ 148,  263,  387,  516,  636,  751,  865, 1000])  # may vary

        >>> X = stats.Uniform(a=np.zeros((3, 1)), b=np.ones(2))
        >>> X.a.shape,
        (3, 2)
        >>> x = X.sample(shape=(5, 4))
        >>> x.shape
        (5, 4, 3, 2)

        """
        raise NotImplementedError()

    @abstractmethod
    def moment(self, order, kind, *, method):
        r"""Raw, central, or standard moment of positive integer order.

        In terms of probability density function :math:`f(x)` and support
        :math:`\chi`, the "raw" moment (about the origin) of order :math:`n` of
        a random variable :math:`X` is:

        .. math::

            \mu'_n(X) = \int_{\chi} x^n f(x) dx

        The "central" moment is the raw moment taken about the mean,
        :math:`\mu = \mu'_1`:

        .. math::

            \mu_n(X) = \int_{\chi} (x - \mu) ^n f(x) dx

        The "standardized" moment is the central moment normalized by the
        :math:`n^\text{th}` power of the standard deviation
        :math:`\sigma = \sqrt{\mu_2}` to produce a scale invariant quantity:

        .. math::

            \tilde{\mu}_n(X) = \frac{\mu_n(X)}
                                    {\sigma^n}

        Parameters
        ----------
        order : int
            The integer order of the moment; i.e. :math:`n` in the formulae above.
        kind : {'raw', 'central', 'standardized'}
            Whether to return the raw (default), central, or standardized moment
            defined above.
        method : {None, 'formula', 'general', 'transform', 'normalize', 'quadrature', 'cache'}
            The strategy used to evaluate the moment. By default (``None``),
            the infrastructure chooses between the following options,
            listed in order of precedence.

            - ``'cache'``: use the value of the moment most recently calculated
              via another method
            - ``'formula'``: use a formula for the moment itself
            - ``'general'``: use a general result that is true for all distributions
              with finite moments; for instance, the zeroth raw moment is
              identically 1
            - ``'transform'``: transform a raw moment to a central moment or
              vice versa (see Notes)
            - ``'normalize'``: normalize a central moment to get a standardized
              or vice versa
            - ``'quadrature'``: numerically integrate according to the definition

            Not all `method` options are available for all orders, kinds, and
            distributions. If the selected `method` is not available, a
            ``NotImplementedError`` will be raised.

        Returns
        -------
        out : array
            The moment of the random variable of the specified order and kind.

        See Also
        --------
        pdf
        mean
        variance
        standard_deviation
        skewness
        kurtosis

        Notes
        -----
        Not all distributions have finite moments of all orders; moments of some
        orders may be undefined or infinite. If a formula for the moment is not
        specifically implemented for the chosen distribution, SciPy will attempt
        to compute the moment via a generic method, which may yield a finite
        result where none exists. This is not a critical bug, but an opportunity
        for an enhancement.

        The definition of a raw moment in the summary is specific to the raw moment
        about the origin. The raw moment about any point :math:`a` is:

        .. math::

            E[(X-a)^n] = \int_{\chi} (x-a)^n f(x) dx

        In this notation, a raw moment about the origin is :math:`\mu'_n = E[x^n]`,
        and a central moment is :math:`\mu_n = E[(x-\mu)^n]`, where :math:`\mu`
        is the first raw moment; i.e. the mean.

        The ``'transform'`` method takes advantage of the following relationships
        between moments taken about different points :math:`a` and :math:`b`.

        .. math::

            E[(X-b)^n] =  \sum_{i=0}^n E[(X-a)^i] {n \choose i} (a - b)^{n-i}

        For instance, to transform the raw moment to the central moment, we let
        :math:`b = \mu` and :math:`a = 0`.

        The distribution infrastructure provides flexibility for distribution
        authors to implement separate formulas for raw moments, central moments,
        and standardized moments of any order. By default, the moment of the
        desired order and kind is evaluated from the formula if such a formula
        is available; if not, the infrastructure uses any formulas that are
        available rather than resorting directly to numerical integration.
        For instance, if formulas for the first three raw moments are
        available and the third standardized moments is desired, the
        infrastructure will evaluate the raw moments and perform the transforms
        and standardization required. The decision tree is somewhat complex,
        but the strategy for obtaining a moment of a given order and kind
        (possibly as an intermediate step due to the recursive nature of the
        transform formula above) roughly follows this order of priority:

        #. Use cache (if order of same moment and kind has been calculated)
        #. Use formula (if available)
        #. Transform between raw and central moment and/or normalize to convert
           between central and standardized moments (if efficient)
        #. Use a generic result true for most distributions (if available)
        #. Use quadrature

        References
        ----------
        .. [1] Moment, *Wikipedia*,
               https://en.wikipedia.org/wiki/Moment_(mathematics)

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the first raw moment:

        >>> X.moment(order=1, kind='raw')
        1.0
        >>> X.moment(order=1, kind='raw') == X.mean() == X.mu
        True

        Evaluate the second central moment:

        >>> X.moment(order=2, kind='central')
        4.0
        >>> X.moment(order=2, kind='central') == X.variance() == X.sigma**2
        True

        Evaluate the fourth standardized moment:

        >>> X.moment(order=4, kind='standardized')
        3.0
        >>> X.moment(order=4, kind='standardized') == X.kurtosis(convention='non-excess')
        True

        """  # noqa:E501
        raise NotImplementedError()

    @abstractmethod
    def mean(self, *, method):
        r"""Mean (raw first moment about the origin)

        Parameters
        ----------
        method : {None, 'formula', 'transform', 'quadrature', 'cache'}
            Method used to calculate the raw first moment. Not
            all methods are available for all distributions. See
            `moment` for details.

        See Also
        --------
        moment
        median
        mode

        References
        ----------
        .. [1] Mean, *Wikipedia*,
               https://en.wikipedia.org/wiki/Mean#Mean_of_a_probability_distribution

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the variance:

        >>> X.mean()
        1.0
        >>> X.mean() == X.moment(order=1, kind='raw') == X.mu
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def median(self, *, method):
        r"""Median (50th percentil)

        If a continuous random variable :math:`X` has probability :math:`0.5` of
        taking on a value less than :math:`m`, then :math:`m` is the median.
        That is, the median is the value :math:`m` for which:

        .. math::

            P(X ≤ m) = 0.5 = P(X ≥ m)

        Parameters
        ----------
        method : {None, 'formula', 'icdf'}
            The strategy used to evaluate the median.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the median
            - ``'icdf'``: evaluate the inverse CDF of 0.5

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The median

        See Also
        --------
        mean
        mode
        icdf

        References
        ----------
        .. [1] Median, *Wikipedia*,
               https://en.wikipedia.org/wiki/Median#Probability_distributions

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Uniform(a=0., b=10.)

        Compute the median:

        >>> X.median()
        np.float64(5.0)
        >>> X.median() == X.icdf(0.5) == X.iccdf(0.5)
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def mode(self, *, method):
        r"""Mode (most likely value)

        Informally, the mode is a value that a random variable has the highest
        probability (density) of assuming. That is, the mode is the element of
        the support :math:`\chi` that maximizes the probability density
        function :math:`f(x)`:

        .. math::

            \text{mode} = \arg\max_{x \in \chi} f(x)

        Parameters
        ----------
        method : {None, 'formula', 'optimization'}
            The strategy used to evaluate the mode.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the median
            - ``'optimization'``: numerically maximize the PDF

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The mode

        See Also
        --------
        mean
        median
        pdf

        Notes
        -----
        For some distributions

        #. the mode is not unique (e.g. the uniform distribution);
        #. the PDF has one or more singularities, and it is debateable whether
           a singularity is considered to be in the domain and called the mode
           (e.g. the gamma distribution with shape parameter less than 1); and/or
        #. the probability density function may have one or more local maxima
           that are not a global maximum (e.g. mixture distributions).

        In such cases, `mode` will

        #. return a single value,
        #. consider the mode to occur at a singularity, and/or
        #. return a local maximum which may or may not be a global maximum.

        If a formula for the mode is not specifically implemented for the
        chosen distribution, SciPy will attempt to compute the mode
        numerically, which may not meet the user's preferred definition of a
        mode. In such cases, the user is encouraged to subclass the
        distribution and override ``mode``.

        References
        ----------
        .. [1] Mode (statistics), *Wikipedia*,
               https://en.wikipedia.org/wiki/Mode_(statistics)

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the mode:

        >>> X.mode()
        1.0

        If the mode is not uniquely defined, ``mode`` nonetheless returns a
        single value.

        >>> X = stats.Uniform(a=0., b=1.)
        >>> X.mode()
        0.5

        If this choice does not satisfy your requirements, subclass the
        distribution and override ``mode``:

        >>> class BetterUniform(stats.Uniform):
        ...     def mode(self):
        ...         return self.b
        >>> X = BetterUniform(a=0., b=1.)
        >>> X.mode()
        1.0

        """
        raise NotImplementedError()

    @abstractmethod
    def variance(self, *, method):
        r"""Variance (central second moment)

        Parameters
        ----------
        method : {None, 'formula', 'transform', 'normalize', 'quadrature', 'cache'}
            Method used to calculate the central second moment. Not
            all methods are available for all distributions. See
            `moment` for details.

        See Also
        --------
        moment
        standard_deviation
        mean

        References
        ----------
        .. [1] Variance, *Wikipedia*,
               https://en.wikipedia.org/wiki/Variance#Absolutely_continuous_random_variable

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the variance:

        >>> X.variance()
        4.0
        >>> X.variance() == X.moment(order=2, kind='central') == X.sigma**2
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def standard_deviation(self, *, method):
        r"""Standard deviation (square root of the second central moment)

        Parameters
        ----------
        method : {None, 'formula', 'transform', 'normalize', 'quadrature', 'cache'}
            Method used to calculate the central second moment. Not
            all methods are available for all distributions. See
            `moment` for details.

        See Also
        --------
        variance
        mean
        moment

        References
        ----------
        .. [1] Standard deviation, *Wikipedia*,
               https://en.wikipedia.org/wiki/Standard_deviation#Definition_of_population_values

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the standard deviation:

        >>> X.standard_deviation()
        2.0
        >>> X.standard_deviation() == X.moment(order=2, kind='central')**0.5 == X.sigma
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def skewness(self, *, method):
        r"""Skewness (standardized third moment)

        Parameters
        ----------
        method : {None, 'formula', 'general', 'transform', 'normalize', 'cache'}
            Method used to calculate the standardized third moment. Not
            all methods are available for all distributions. See
            `moment` for details.

        See Also
        --------
        moment
        mean
        variance

        References
        ----------
        .. [1] Skewness, *Wikipedia*,
               https://en.wikipedia.org/wiki/Skewness

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the skewness:

        >>> X.skewness()
        0.0
        >>> X.skewness() == X.moment(order=3, kind='standardized')
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def kurtosis(self, *, method):
        r"""Kurtosis (standardized fourth moment)

        By default, this is the standardized fourth moment, also known as the
        "non-excess" or "Pearson" kurtosis (e.g. the kurtosis of the normal
        distribution is 3). The "excess" or "Fisher" kurtosis (the standardized
        fourth moment minus 3) is available via the `convention` parameter.

        Parameters
        ----------
        method : {None, 'formula', 'general', 'transform', 'normalize', 'cache'}
            Method used to calculate the standardized fourth moment. Not
            all methods are available for all distributions. See
            `moment` for details.
        convention : {'non-excess', 'excess'}
            Two distinct conventions are available:

            - ``'non-excess'``: the standardized fourth moment (Pearson's kurtosis)
            - ``'excess'``: the standardized fourth moment minus 3 (Fisher's kurtosis)

            The default is ``'non-excess'``.

        See Also
        --------
        moment
        mean
        variance

        References
        ----------
        .. [1] Kurtosis, *Wikipedia*,
               https://en.wikipedia.org/wiki/Kurtosis

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Normal(mu=1., sigma=2.)

        Evaluate the kurtosis:

        >>> X.kurtosis()
        3.0
        >>> (X.kurtosis()
        ...  == X.kurtosis(convention='excess') + 3.
        ...  == X.moment(order=4, kind='standardized'))
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def pdf(self, x, /, *, method):
        r"""Probability density function

        The probability density function ("PDF"), denoted :math:`f(x)`, is the
        probability *per unit length* that the random variable will assume the
        value :math:`x`. Mathematically, it can be defined as the derivative
        of the cumulative distribution function :math:`F(x)`:

        .. math::

            f(x) = \frac{d}{dx} F(x)

        `pdf` accepts `x` for :math:`x`.

        Parameters
        ----------
        x : array_like
            The argument of the PDF.
        method : {None, 'formula', 'logexp'}
            The strategy used to evaluate the PDF. By default (``None``), the
            infrastructure chooses between the following options, listed in
            order of precedence.

            - ``'formula'``: use a formula for the PDF itself
            - ``'logexp'``: evaluate the log-PDF and exponentiate

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The PDF evaluated at the argument `x`.

        See Also
        --------
        cdf
        logpdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        By definition of the support, the PDF evaluates to its minimum value
        of :math:`0` outside the support; i.e. for :math:`x < l` or
        :math:`x > r`. The maximum of the PDF may be less than or greater than
        :math:`1`; since the valus is a probability *density*, only its integral
        over the support must equal :math:`1`.

        References
        ----------
        .. [1] Probability density function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Probability_density_function

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Uniform(a=-1., b=1.)

        Evaluate the PDF at the desired argument:

        >>> X.pdf(0.25)
        0.5

        """
        raise NotImplementedError()

    @abstractmethod
    def logpdf(self, x, /, *, method):
        r"""Log of the probability density function

        The probability density function ("PDF"), denoted :math:`f(x)`, is the
        probability *per unit length* that the random variable will assume the
        value :math:`x`. Mathematically, it can be defined as the derivative
        of the cumulative distribution function :math:`F(x)`:

        .. math::

            f(x) = \frac{d}{dx} F(x)

        `logpdf` computes the logarithm of the probability density function
        ("log-PDF"), :math:`\log(f(x))`, but it may be numerically favorable
        compared to the naive implementation (computing :math:`f(x)` and
        taking the logarithm).

        `logpdf` accepts `x` for :math:`x`.

        Parameters
        ----------
        x : array_like
            The argument of the log-PDF.
        method : {None, 'formula', 'logexp'}
            The strategy used to evaluate the log-PDF. By default (``None``), the
            infrastructure chooses between the following options, listed in order
            of precedence.

            - ``'formula'``: use a formula for the log-PDF itself
            - ``'logexp'``: evaluate the PDF and takes its logarithm

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The log-PDF evaluated at the argument `x`.

        See Also
        --------
        pdf
        logcdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        By definition of the support, the log-PDF evaluates to its minimum value
        of :math:`-\infty` (i.e. :math:`\log(0)`) outside the support; i.e. for
        :math:`x < l` or :math:`x > r`. The maximum of the log-PDF may be less
        than or greater than :math:`\log(1) = 0` because the maximum of the PDF
        can be any positive real.

        For distributions with infinite support, it is common for `pdf` to return
        a value of ``0`` when the argument is theoretically within the support;
        this can occur because the true value of the PDF is too small to be
        represented by the chosen dtype. The log-PDF, however, will often be finite
        (not ``-inf``) over a much larger domain. Consequently, it may be preferred
        to work with the logarithms of probabilities and probability densities to
        avoid underflow.

        References
        ----------
        .. [1] Probability density function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Probability_density_function

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-1.0, b=1.0)

        Evaluate the log-PDF at the desired argument:

        >>> X.logpdf(0.5)
        -0.6931471805599453
        >>> np.allclose(X.logpdf(0.5), np.log(X.pdf(0.5)))
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def cdf(self, x, y, /, *, method):
        r"""Cumulative distribution function

        The cumulative distribution function ("CDF"), denoted :math:`F(x)`, is
        the probability the random variable :math:`X` will assume a value
        less than or equal to :math:`x`:

        .. math::

            F(x) = P(X ≤ x)

        A two-argument variant of this function is also defined as the
        probability the random variable :math:`X` will assume a value between
        :math:`x` and :math:`y`.

        .. math::

            F(x, y) = P(x ≤ X ≤ y)

        `cdf` accepts `x` for :math:`x` and `y` for :math:`y`.

        Parameters
        ----------
        x, y : array_like
            The arguments of the CDF. `x` is required; `y` is optional.
        method : {None, 'formula', 'logexp', 'complement', 'quadrature', 'subtraction'}
            The strategy used to evaluate the CDF.
            By default (``None``), the one-argument form of the function
            chooses between the following options, listed in order of precedence.

            - ``'formula'``: use a formula for the CDF itself
            - ``'logexp'``: evaluate the log-CDF and exponentiate
            - ``'complement'``: evaluate the CCDF and take the complement
            - ``'quadrature'``: numerically integrate the PDF

            In place of ``'complement'``, the two-argument form accepts:

            - ``'subtraction'``: compute the CDF at each argument and take
              the difference.

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The CDF evaluated at the provided argument(s).

        See Also
        --------
        logcdf
        ccdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        The CDF :math:`F(x)` is related to the probability density function
        :math:`f(x)` by:

        .. math::

            F(x) = \int_l^x f(u) du

        The two argument version is:

        .. math::

            F(x, y) = \int_x^y f(u) du = F(y) - F(x)

        The CDF evaluates to its minimum value of :math:`0` for :math:`x ≤ l`
        and its maximum value of :math:`1` for :math:`x ≥ r`.

        The CDF is also known simply as the "distribution function".

        References
        ----------
        .. [1] Cumulative distribution function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Cumulative_distribution_function

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the CDF at the desired argument:

        >>> X.cdf(0.25)
        0.75

        Evaluate the cumulative probability between two arguments:

        >>> X.cdf(-0.25, 0.25) == X.cdf(0.25) - X.cdf(-0.25)
        True

        """  # noqa: E501
        raise NotImplementedError()

    @abstractmethod
    def icdf(self, p, /, *, method):
        r"""Inverse of the cumulative distribution function.

        The inverse of the cumulative distribution function ("inverse CDF"),
        denoted :math:`F^{-1}(p)`, is the argument :math:`x` for which the
        cumulative distribution function :math:`F(x)` evaluates to :math:`p`.

        .. math::

            F^{-1}(p) = x \quad \text{s.t.} \quad F(x) = p

        `icdf` accepts `p` for :math:`p \in [0, 1]`.

        Parameters
        ----------
        p : array_like
            The argument of the inverse CDF.
        method : {None, 'formula', 'complement', 'inversion'}
            The strategy used to evaluate the inverse CDF.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the inverse CDF itself
            - ``'complement'``: evaluate the inverse CCDF at the
              complement of `p`
            - ``'inversion'``: solve numerically for the argument at which the
              CDF is equal to `p`

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The inverse CDF evaluated at the provided argument.

        See Also
        --------
        cdf
        ilogcdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`. The
        inverse CDF returns its minimum value of :math:`l` at :math:`p = 0`
        and its maximum value of :math:`r` at :math:`p = 1`. Because the CDF
        has range :math:`[0, 1]`, the inverse CDF is only defined on the
        domain :math:`[0, 1]`; for :math:`p < 0` and :math:`p > 1`, `icdf`
        returns ``nan``.

        The inverse CDF is also known as the quantile function, percentile function,
        and percent-point function.

        References
        ----------
        .. [1] Quantile function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Quantile_function

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the inverse CDF at the desired argument:

        >>> X.icdf(0.25)
        -0.25
        >>> np.allclose(X.cdf(X.icdf(0.25)), 0.25)
        True

        This function returns NaN when the argument is outside the domain.

        >>> X.icdf([-0.1, 0, 1, 1.1])
        array([ nan, -0.5,  0.5,  nan])

        """
        raise NotImplementedError()

    @abstractmethod
    def ccdf(self, x, y, /, *, method):
        r"""Complementary cumulative distribution function

        The complementary cumulative distribution function ("CCDF"), denoted
        :math:`G(x)`, is the complement of the cumulative distribution function
        :math:`F(x)`; i.e., probability the random variable :math:`X` will
        assume a value greater than :math:`x`:

        .. math::

            G(x) = 1 - F(x) = P(X > x)

        A two-argument variant of this function is:

        .. math::

            G(x, y) = 1 - F(x, y) = P(X < x \text{ or } X > y)

        `ccdf` accepts `x` for :math:`x` and `y` for :math:`y`.

        Parameters
        ----------
        x, y : array_like
            The arguments of the CCDF. `x` is required; `y` is optional.
        method : {None, 'formula', 'logexp', 'complement', 'quadrature', 'addition'}
            The strategy used to evaluate the CCDF.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the CCDF itself
            - ``'logexp'``: evaluate the log-CCDF and exponentiate
            - ``'complement'``: evaluate the CDF and take the complement
            - ``'quadrature'``: numerically integrate the PDF

            The two-argument form chooses between:

            - ``'formula'``: use a formula for the CCDF itself
            - ``'addition'``: compute the CDF at `x` and the CCDF at `y`, then add

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The CCDF evaluated at the provided argument(s).

        See Also
        --------
        cdf
        logccdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        The CCDF :math:`G(x)` is related to the probability density function
        :math:`f(x)` by:

        .. math::

            G(x) = \int_x^r f(u) du

        The two argument version is:

        .. math::

            G(x, y) = \int_l^x f(u) du + \int_y^r f(u) du

        The CCDF returns its minimum value of :math:`0` for :math:`x ≥ r`
        and its maximum value of :math:`1` for :math:`x ≤ l`.

        The CCDF is also known as the "survival function".

        References
        ----------
        .. [1] Cumulative distribution function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Cumulative_distribution_function#Derived_functions

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the CCDF at the desired argument:

        >>> X.ccdf(0.25)
        0.25
        >>> np.allclose(X.ccdf(0.25), 1-X.cdf(0.25))
        True

        Evaluate the complement of the cumulative probability between two arguments:

        >>> X.ccdf(-0.25, 0.25) == X.cdf(-0.25) + X.ccdf(0.25)
        True

        """  # noqa: E501
        raise NotImplementedError()

    @abstractmethod
    def iccdf(self, p, /, *, method):
        r"""Inverse complementary cumulative distribution function.

        The inverse complementary cumulative distribution function ("inverse CCDF"),
        denoted :math:`G^{-1}(p)`, is the argument :math:`x` for which the
        complementary cumulative distribution function :math:`G(x)` evaluates to
        :math:`p`.

        .. math::

            G^{-1}(p) = x \quad \text{s.t.} \quad G(x) = p

        `iccdf` accepts `p` for :math:`p \in [0, 1]`.

        Parameters
        ----------
        p : array_like
            The argument of the inverse CCDF.
        method : {None, 'formula', 'complement', 'inversion'}
            The strategy used to evaluate the inverse CCDF.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the inverse CCDF itself
            - ``'complement'``: evaluate the inverse CDF at the
              complement of `p`
            - ``'inversion'``: solve numerically for the argument at which the
              CCDF is equal to `p`

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The inverse CCDF evaluated at the provided argument.

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`. The
        inverse CCDF returns its minimum value of :math:`l` at :math:`p = 1`
        and its maximum value of :math:`r` at :math:`p = 0`. Because the CCDF
        has range :math:`[0, 1]`, the inverse CCDF is only defined on the
        domain :math:`[0, 1]`; for :math:`p < 0` and :math:`p > 1`, ``iccdf``
        returns ``nan``.

        See Also
        --------
        icdf
        ilogccdf

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the inverse CCDF at the desired argument:

        >>> X.iccdf(0.25)
        0.25
        >>> np.allclose(X.iccdf(0.25), X.icdf(1-0.25))
        True

        This function returns NaN when the argument is outside the domain.

        >>> X.iccdf([-0.1, 0, 1, 1.1])
        array([ nan,  0.5, -0.5,  nan])

        """
        raise NotImplementedError()

    @abstractmethod
    def logcdf(self, x, y, /, *, method):
        r"""Log of the cumulative distribution function

        The cumulative distribution function ("CDF"), denoted :math:`F(x)`, is
        the probability the random variable :math:`X` will assume a value
        less than or equal to :math:`x`:

        .. math::

            F(x) = P(X ≤ x)

        A two-argument variant of this function is also defined as the
        probability the random variable :math:`X` will assume a value between
        :math:`x` and :math:`y`.

        .. math::

            F(x, y) = P(x ≤ X ≤ y)

        `logcdf` computes the logarithm of the cumulative distribution function
        ("log-CDF"), :math:`\log(F(x))`/:math:`\log(F(x, y))`, but it may be
        numerically favorable compared to the naive implementation (computing
        the CDF and taking the logarithm).

        `logcdf` accepts `x` for :math:`x` and `y` for :math:`y`.

        Parameters
        ----------
        x, y : array_like
            The arguments of the log-CDF. `x` is required; `y` is optional.
        method : {None, 'formula', 'logexp', 'complement', 'quadrature', 'subtraction'}
            The strategy used to evaluate the log-CDF.
            By default (``None``), the one-argument form of the function
            chooses between the following options, listed in order of precedence.

            - ``'formula'``: use a formula for the log-CDF itself
            - ``'logexp'``: evaluate the CDF and take the logarithm
            - ``'complement'``: evaluate the log-CCDF and take the
              logarithmic complement (see Notes)
            - ``'quadrature'``: numerically log-integrate the log-PDF

            In place of ``'complement'``, the two-argument form accepts:

            - ``'subtraction'``: compute the log-CDF at each argument and take
              the logarithmic difference (see Notes)

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The log-CDF evaluated at the provided argument(s).

        See Also
        --------
        cdf
        logccdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        The log-CDF evaluates to its minimum value of :math:`\log(0) = -\infty`
        for :math:`x ≤ l` and its maximum value of :math:`\log(1) = 0` for
        :math:`x ≥ r`.

        For distributions with infinite support, it is common for
        `cdf` to return a value of ``0`` when the argument
        is theoretically within the support; this can occur because the true value
        of the CDF is too small to be represented by the chosen dtype. `logcdf`,
        however, will often return a finite (not ``-inf``) result over a much larger
        domain. Similarly, `logcdf` may provided a strictly negative result with
        arguments for which `cdf` would return ``1.0``. Consequently, it may be
        preferred to work with the logarithms of probabilities to avoid underflow
        and related limitations of floating point numbers.

        The "logarithmic complement" of a number :math:`z` is mathematically
        equivalent to :math:`\log(1-\exp(z))`, but it is computed to avoid loss
        of precision when :math:`\exp(z)` is nearly :math:`0` or :math:`1`.
        Similarly, the term "logarithmic difference" of :math:`w` and :math:`z`
        is used here to mean :math:`\log(\exp(w)-\exp(z))`.

        If ``y < x``, the CDF is negative, and therefore the log-CCDF
        is complex with imaginary part :math:`\pi`. For
        consistency, the result of this function always has complex dtype
        when `y` is provided, regardless of the value of the imaginary part.

        References
        ----------
        .. [1] Cumulative distribution function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Cumulative_distribution_function

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the log-CDF at the desired argument:

        >>> X.logcdf(0.25)
        -0.287682072451781
        >>> np.allclose(X.logcdf(0.), np.log(X.cdf(0.)))
        True

        """  # noqa: E501
        raise NotImplementedError()

    @abstractmethod
    def ilogcdf(self, logp, /, *, method):
        r"""Inverse of the logarithm of the cumulative distribution function.

        The inverse of the logarithm of the cumulative distribution function
        ("inverse log-CDF") is the argument :math:`x` for which the logarithm
        of the cumulative distribution function :math:`\log(F(x))` evaluates
        to :math:`\log(p)`.

        Mathematically, it is equivalent to :math:`F^{-1}(\exp(y))`, where
        :math:`y = \log(p)`, but it may be numerically favorable compared to
        the naive implementation (computing :math:`p = \exp(y)`, then
        :math:`F^{-1}(p)`).

        `ilogcdf` accepts `logp` for :math:`\log(p) ≤ 0`.

        Parameters
        ----------
        logp : array_like
            The argument of the inverse log-CDF.
        method : {None, 'formula', 'complement', 'inversion'}
            The strategy used to evaluate the inverse log-CDF.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the inverse log-CDF itself
            - ``'complement'``: evaluate the inverse log-CCDF at the
              logarithmic complement of `logp` (see Notes)
            - ``'inversion'``: solve numerically for the argument at which the
              log-CDF is equal to `logp`

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The inverse log-CDF evaluated at the provided argument.

        See Also
        --------
        icdf
        logcdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        The inverse log-CDF returns its minimum value of :math:`l` at
        :math:`\log(p) = \log(0) = -\infty` and its maximum value of :math:`r` at
        :math:`\log(p) = \log(1) = 0`. Because the log-CDF has range
        :math:`[-\infty, 0]`, the inverse log-CDF is only defined on the
        negative reals; for :math:`\log(p) > 0`, `ilogcdf` returns ``nan``.

        Occasionally, it is needed to find the argument of the CDF for which
        the resulting probability is very close to ``0`` or ``1`` - too close to
        represent accurately with floating point arithmetic. In many cases,
        however, the *logarithm* of this resulting probability may be
        represented in floating point arithmetic, in which case this function
        may be used to find the argument of the CDF for which the *logarithm*
        of the resulting probability is :math:`y = \log(p)`.

        The "logarithmic complement" of a number :math:`z` is mathematically
        equivalent to :math:`\log(1-\exp(z))`, but it is computed to avoid loss
        of precision when :math:`\exp(z)` is nearly :math:`0` or :math:`1`.

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the inverse log-CDF at the desired argument:

        >>> X.ilogcdf(-0.25)
        0.2788007830714034
        >>> np.allclose(X.ilogcdf(-0.25), X.icdf(np.exp(-0.25)))
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def logccdf(self, x, y, /, *, method):
        r"""Log of the complementary cumulative distribution function

        The complementary cumulative distribution function ("CCDF"), denoted
        :math:`G(x)` is the complement of the cumulative distribution function
        :math:`F(x)`; i.e., probability the random variable :math:`X` will
        assume a value greater than :math:`x`:

        .. math::

            G(x) = 1 - F(x) = P(X > x)

        A two-argument variant of this function is:

        .. math::

            G(x, y) = 1 - F(x, y) = P(X < x \quad \text{or} \quad X > y)

        `logccdf` computes the logarithm of the complementary cumulative
        distribution function ("log-CCDF"), :math:`\log(G(x))`/:math:`\log(G(x, y))`,
        but it may be numerically favorable compared to the naive implementation
        (computing the CDF and taking the logarithm).

        `logccdf` accepts `x` for :math:`x` and `y` for :math:`y`.

        Parameters
        ----------
        x, y : array_like
            The arguments of the log-CCDF. `x` is required; `y` is optional.
        method : {None, 'formula', 'logexp', 'complement', 'quadrature', 'addition'}
            The strategy used to evaluate the log-CCDF.
            By default (``None``), the one-argument form of the function
            chooses between the following options, listed in order of precedence.

            - ``'formula'``: use a formula for the log CCDF itself
            - ``'logexp'``: evaluate the CCDF and take the logarithm
            - ``'complement'``: evaluate the log-CDF and take the
              logarithmic complement (see Notes)
            - ``'quadrature'``: numerically log-integrate the log-PDF

            The two-argument form chooses between:

            - ``'formula'``: use a formula for the log CCDF itself
            - ``'addition'``: compute the log-CDF at `x` and the log-CCDF at `y`,
              then take the logarithmic sum (see Notes)

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The log-CCDF evaluated at the provided argument(s).

        See Also
        --------
        ccdf
        logcdf

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`.
        The log-CCDF returns its minimum value of :math:`\log(0)=-\infty` for
        :math:`x ≥ r` and its maximum value of :math:`\log(1) = 0` for
        :math:`x ≤ l`.

        For distributions with infinite support, it is common for
        `ccdf` to return a value of ``0`` when the argument
        is theoretically within the support; this can occur because the true value
        of the CCDF is too small to be represented by the chosen dtype. The log
        of the CCDF, however, will often be finite (not ``-inf``) over a much larger
        domain. Similarly, `logccdf` may provided a strictly negative result with
        arguments for which `ccdf` would return ``1.0``. Consequently, it may be
        preferred to work with the logarithms of probabilities to avoid underflow
        and related limitations of floating point numbers.

        The "logarithmic complement" of a number :math:`z` is mathematically
        equivalent to :math:`\log(1-\exp(z))`, but it is computed to avoid loss
        of precision when :math:`\exp(z)` is nearly :math:`0` or :math:`1`.
        Similarly, the term "logarithmic sum" of :math:`w` and :math:`z`
        is used here to mean the :math:`\log(\exp(w)+\exp(z))`, AKA
        :math:`\text{LogSumExp}(w, z)`.

        References
        ----------
        .. [1] Cumulative distribution function, *Wikipedia*,
               https://en.wikipedia.org/wiki/Cumulative_distribution_function#Derived_functions

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the log-CCDF at the desired argument:

        >>> X.logccdf(0.25)
        -1.3862943611198906
        >>> np.allclose(X.logccdf(0.), np.log(X.ccdf(0.)))
        True

        """  # noqa: E501
        raise NotImplementedError()

    @abstractmethod
    def ilogccdf(self, logp, /, *, method):
        r"""Inverse of the log of the complementary cumulative distribution function.

        The inverse of the logarithm of the complementary cumulative distribution
        function ("inverse log-CCDF") is the argument :math:`x` for which the logarithm
        of the complementary cumulative distribution function :math:`\log(G(x))`
        evaluates to :math:`\log(p)`.

        Mathematically, it is equivalent to :math:`G^{-1}(\exp(y))`, where
        :math:`y = \log(p)`, but it may be numerically favorable compared to the naive
        implementation (computing :math:`p = \exp(y)`, then :math:`G^{-1}(p)`).

        `ilogccdf` accepts `logp` for :math:`\log(p) ≤ 0`.

        Parameters
        ----------
        x : array_like
            The argument of the inverse log-CCDF.
        method : {None, 'formula', 'complement', 'inversion'}
            The strategy used to evaluate the inverse log-CCDF.
            By default (``None``), the infrastructure chooses between the
            following options, listed in order of precedence.

            - ``'formula'``: use a formula for the inverse log-CCDF itself
            - ``'complement'``: evaluate the inverse log-CDF at the
              logarithmic complement of `x` (see Notes)
            - ``'inversion'``: solve numerically for the argument at which the
              log-CCDF is equal to `x`

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The inverse log-CCDF evaluated at the provided argument.

        Notes
        -----
        Suppose a continuous probability distribution has support :math:`[l, r]`. The
        inverse log-CCDF returns its minimum value of :math:`l` at
        :math:`\log(p) = \log(1) = 0` and its maximum value of :math:`r` at
        :math:`\log(p) = \log(0) = -\infty`. Because the log-CCDF has range
        :math:`[-\infty, 0]`, the inverse log-CDF is only defined on the
        negative reals; for :math:`\log(p) > 0`, `ilogccdf` returns ``nan``.

        Occasionally, it is needed to find the argument of the CCDF for which
        the resulting probability is very close to ``0`` or ``1`` - too close to
        represent accurately with floating point arithmetic. In many cases,
        however, the *logarithm* of this resulting probability may be
        represented in floating point arithmetic, in which case this function
        may be used to find the argument of the CCDF for which the *logarithm*
        of the resulting probability is `y = \log(p)`.

        The "logarithmic complement" of a number :math:`z` is mathematically
        equivalent to :math:`\log(1-\exp(z))`, but it is computed to avoid loss
        of precision when :math:`\exp(z)` is nearly :math:`0` or :math:`1`.

        See Also
        --------
        iccdf
        ilogccdf

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-0.5, b=0.5)

        Evaluate the inverse log-CCDF at the desired argument:

        >>> X.ilogccdf(-0.25)
        -0.2788007830714034
        >>> np.allclose(X.ilogccdf(-0.25), X.iccdf(np.exp(-0.25)))
        True

        """
        raise NotImplementedError()

    @abstractmethod
    def logentropy(self, *, method):
        r"""Logarithm of the differential entropy

        In terms of probability density function :math:`f(x)` and support
        :math:`\chi`, the differential entropy (or simply "entropy") of a random
        variable :math:`X` is:

        .. math::

            h(X) = - \int_{\chi} f(x) \log f(x) dx

        `logentropy` computes the logarithm of the differential entropy
        ("log-entropy"), :math:`log(h(X))`, but it may be numerically favorable
        compared to the naive implementation (computing :math:`h(X)` then
        taking the logarithm).

        Parameters
        ----------
        method : {None, 'formula', 'logexp', 'quadrature}
            The strategy used to evaluate the log-entropy. By default
            (``None``), the infrastructure chooses between the following options,
            listed in order of precedence.

            - ``'formula'``: use a formula for the log-entropy itself
            - ``'logexp'``: evaluate the entropy and take the logarithm
            - ``'quadrature'``: numerically log-integrate the logarithm of the
              entropy integrand

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The log-entropy.

        See Also
        --------
        entropy
        logpdf

        Notes
        -----
        If the entropy of a distribution is negative, then the log-entropy
        is complex with imaginary part :math:`\pi`. For
        consistency, the result of this function always has complex dtype,
        regardless of the value of the imaginary part.

        References
        ----------
        .. [1] Differential entropy, *Wikipedia*,
               https://en.wikipedia.org/wiki/Differential_entropy

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> import numpy as np
        >>> from scipy import stats
        >>> X = stats.Uniform(a=-1., b=1.)

        Evaluate the log-entropy:

        >>> X.logentropy()
        (-0.3665129205816642+0j)
        >>> np.allclose(np.exp(X.logentropy()), X.entropy())
        True

        For a random variable with negative entropy, the log-entropy has an
        imaginary part equal to `np.pi`.

        >>> X = stats.Uniform(a=-.1, b=.1)
        >>> X.entropy(), X.logentropy()
        (-1.6094379124341007, (0.4758849953271105+3.141592653589793j))

        """
        raise NotImplementedError()
        
    @abstractmethod
    def entropy(self, *, method):
        r"""Differential entropy

        In terms of probability density function :math:`f(x)` and support
        :math:`\chi`, the differential entropy (or simply "entropy") of a
        continuous random variable :math:`X` is:

        .. math::

            h(X) = - \int_{\chi} f(x) \log f(x) dx

        Parameters
        ----------
        method : {None, 'formula', 'logexp', 'quadrature'}
            The strategy used to evaluate the entropy. By default (``None``),
            the infrastructure chooses between the following options, listed
            in order of precedence.

            - ``'formula'``: use a formula for the entropy itself
            - ``'logexp'``: evaluate the log-entropy and exponentiate
            - ``'quadrature'``: use numerical integration

            Not all `method` options are available for all distributions.
            If the selected `method` is not available, a ``NotImplementedError``
            will be raised.

        Returns
        -------
        out : array
            The entropy of the random variable.

        See Also
        --------
        logentropy
        pdf

        Notes
        -----
        This function calculates the entropy using the natural logarithm; i.e.
        the logarithm with base :math:`e`. Consequently, the value is expressed
        in (dimensionless) "units" of nats. To convert the entropy to different
        units (i.e. corresponding with a different base), divide the result by
        the natural logarithm of the desired base.

        References
        ----------
        .. [1] Differential entropy, *Wikipedia*,
               https://en.wikipedia.org/wiki/Differential_entropy

        Examples
        --------
        Instantiate a distribution with the desired parameters:

        >>> from scipy import stats
        >>> X = stats.Uniform(a=-1., b=1.)

        Evaluate the entropy:

        >>> X.entropy()
        0.6931471805599454

        """
        raise NotImplementedError()



# Pytest customization
import json
import os
import warnings
import tempfile
from contextlib import contextmanager

import numpy as np
import numpy.testing as npt
import pytest
import hypothesis

from scipy._lib._fpumode import get_fpu_mode
from scipy._lib._testutils import FPUModeChangeWarning
from scipy._lib._array_api import SCIPY_ARRAY_API, SCIPY_DEVICE
from scipy._lib import _pep440

try:
    from scipy_doctest.conftest import dt_config
    HAVE_SCPDT = True
except ModuleNotFoundError:
    HAVE_SCPDT = False

try:
    import pytest_run_parallel  # noqa:F401
    PARALLEL_RUN_AVAILABLE = True
except Exception:
    PARALLEL_RUN_AVAILABLE = False


def pytest_configure(config):
    config.addinivalue_line("markers",
        "slow: Tests that are very slow.")
    config.addinivalue_line("markers",
        "xslow: mark test as extremely slow (not run unless explicitly requested)")
    config.addinivalue_line("markers",
        "xfail_on_32bit: mark test as failing on 32-bit platforms")
    try:
        import pytest_timeout  # noqa:F401
    except Exception:
        config.addinivalue_line(
            "markers", 'timeout: mark a test for a non-default timeout')
    try:
        # This is a more reliable test of whether pytest_fail_slow is installed
        # When I uninstalled it, `import pytest_fail_slow` didn't fail!
        from pytest_fail_slow import parse_duration  # type: ignore[import-not-found] # noqa:F401,E501
    except Exception:
        config.addinivalue_line(
            "markers", 'fail_slow: mark a test for a non-default timeout failure')
    config.addinivalue_line("markers",
        "skip_xp_backends(backends, reason=None, np_only=False, cpu_only=False, "
        "exceptions=None): "
        "mark the desired skip configuration for the `skip_xp_backends` fixture.")
    config.addinivalue_line("markers",
        "xfail_xp_backends(backends, reason=None, np_only=False, cpu_only=False, "
        "exceptions=None): "
        "mark the desired xfail configuration for the `xfail_xp_backends` fixture.")
    if not PARALLEL_RUN_AVAILABLE:
        config.addinivalue_line(
            'markers',
            'parallel_threads(n): run the given test function in parallel '
            'using `n` threads.')
        config.addinivalue_line(
            "markers",
            "thread_unsafe: mark the test function as single-threaded",
        )
        config.addinivalue_line(
            "markers",
            "iterations(n): run the given test function `n` times in each thread",
        )


def pytest_runtest_setup(item):
    mark = item.get_closest_marker("xslow")
    if mark is not None:
        try:
            v = int(os.environ.get('SCIPY_XSLOW', '0'))
        except ValueError:
            v = False
        if not v:
            pytest.skip("very slow test; "
                        "set environment variable SCIPY_XSLOW=1 to run it")
    mark = item.get_closest_marker("xfail_on_32bit")
    if mark is not None and np.intp(0).itemsize < 8:
        pytest.xfail(f'Fails on our 32-bit test platform(s): {mark.args[0]}')

    # Older versions of threadpoolctl have an issue that may lead to this
    # warning being emitted, see gh-14441
    with npt.suppress_warnings() as sup:
        sup.filter(pytest.PytestUnraisableExceptionWarning)

        try:
            from threadpoolctl import threadpool_limits

            HAS_THREADPOOLCTL = True
        except Exception:  # observed in gh-14441: (ImportError, AttributeError)
            # Optional dependency only. All exceptions are caught, for robustness
            HAS_THREADPOOLCTL = False

        if HAS_THREADPOOLCTL:
            # Set the number of openmp threads based on the number of workers
            # xdist is using to prevent oversubscription. Simplified version of what
            # sklearn does (it can rely on threadpoolctl and its builtin OpenMP helper
            # functions)
            try:
                xdist_worker_count = int(os.environ['PYTEST_XDIST_WORKER_COUNT'])
            except KeyError:
                # raises when pytest-xdist is not installed
                return

            if not os.getenv('OMP_NUM_THREADS'):
                max_openmp_threads = os.cpu_count() // 2  # use nr of physical cores
                threads_per_worker = max(max_openmp_threads // xdist_worker_count, 1)
                try:
                    threadpool_limits(threads_per_worker, user_api='blas')
                except Exception:
                    # May raise AttributeError for older versions of OpenBLAS.
                    # Catch any error for robustness.
                    return


@pytest.fixture(scope="function", autouse=True)
def check_fpu_mode(request):
    """
    Check FPU mode was not changed during the test.
    """
    old_mode = get_fpu_mode()
    yield
    new_mode = get_fpu_mode()

    if old_mode != new_mode:
        warnings.warn(f"FPU mode changed from {old_mode:#x} to {new_mode:#x} during "
                      "the test",
                      category=FPUModeChangeWarning, stacklevel=0)


if not PARALLEL_RUN_AVAILABLE:
    @pytest.fixture
    def num_parallel_threads():
        return 1


# Array API backend handling
xp_available_backends = {'numpy': np}

if SCIPY_ARRAY_API and isinstance(SCIPY_ARRAY_API, str):
    # fill the dict of backends with available libraries
    try:
        import array_api_strict
        xp_available_backends.update({'array_api_strict': array_api_strict})
        if _pep440.parse(array_api_strict.__version__) < _pep440.Version('2.0'):
            raise ImportError("array-api-strict must be >= version 2.0")
        array_api_strict.set_array_api_strict_flags(
            api_version='2023.12'
        )
    except ImportError:
        pass

    try:
        import torch  # type: ignore[import-not-found]
        xp_available_backends.update({'torch': torch})
        # can use `mps` or `cpu`
        torch.set_default_device(SCIPY_DEVICE)
    except ImportError:
        pass

    try:
        import cupy  # type: ignore[import-not-found]
        xp_available_backends.update({'cupy': cupy})
    except ImportError:
        pass

    try:
        import jax.numpy  # type: ignore[import-not-found]
        xp_available_backends.update({'jax.numpy': jax.numpy})
        jax.config.update("jax_enable_x64", True)
        jax.config.update("jax_default_device", jax.devices(SCIPY_DEVICE)[0])
    except ImportError:
        pass

    # by default, use all available backends
    if SCIPY_ARRAY_API.lower() not in ("1", "true"):
        SCIPY_ARRAY_API_ = json.loads(SCIPY_ARRAY_API)

        if 'all' in SCIPY_ARRAY_API_:
            pass  # same as True
        else:
            # only select a subset of backend by filtering out the dict
            try:
                xp_available_backends = {
                    backend: xp_available_backends[backend]
                    for backend in SCIPY_ARRAY_API_
                }
            except KeyError:
                msg = f"'--array-api-backend' must be in {xp_available_backends.keys()}"
                raise ValueError(msg)

if 'cupy' in xp_available_backends:
    SCIPY_DEVICE = 'cuda'

array_api_compatible = pytest.mark.parametrize("xp", xp_available_backends.values())

skip_xp_invalid_arg = pytest.mark.skipif(SCIPY_ARRAY_API,
    reason = ('Test involves masked arrays, object arrays, or other types '
              'that are not valid input when `SCIPY_ARRAY_API` is used.'))


def _backends_kwargs_from_request(request, skip_or_xfail):
    """A helper for {skip,xfail}_xp_backends"""
    # do not allow multiple backends
    args_ = request.keywords[f'{skip_or_xfail}_xp_backends'].args
    if len(args_) > 1:
        # np_only / cpu_only has args=(), otherwise it's ('numpy',)
        # and we do not allow ('numpy', 'cupy')
        raise ValueError(f"multiple backends: {args_}")

    markers = list(request.node.iter_markers(f'{skip_or_xfail}_xp_backends'))
    backends = []
    kwargs = {}
    for marker in markers:
        if marker.kwargs.get('np_only'):
            kwargs['np_only'] = True
            kwargs['exceptions'] = marker.kwargs.get('exceptions', [])
        elif marker.kwargs.get('cpu_only'):
            if not kwargs.get('np_only'):
                # if np_only is given, it is certainly cpu only
                kwargs['cpu_only'] = True
                kwargs['exceptions'] = marker.kwargs.get('exceptions', [])

        # add backends, if any
        if len(marker.args) > 0:
            backend = marker.args[0]  # was a tuple, ('numpy',) etc
            backends.append(backend)
            kwargs.update(**{backend: marker.kwargs})

    return backends, kwargs


@pytest.fixture
def skip_xp_backends(xp, request):
    """skip_xp_backends(backend=None, reason=None, np_only=False, cpu_only=False, exceptions=None)

    Skip a decorated test for the provided backend, or skip a category of backends.

    See ``skip_or_xfail_backends`` docstring for details. Note that, contrary to
    ``skip_or_xfail_backends``, the ``backend`` and ``reason`` arguments are optional
    single strings: this function only skips a single backend at a time.
    To skip multiple backends, provide multiple decorators.
    """  # noqa: E501
    if "skip_xp_backends" not in request.keywords:
        return

    backends, kwargs = _backends_kwargs_from_request(request, skip_or_xfail='skip')
    skip_or_xfail_xp_backends(xp, backends, kwargs, skip_or_xfail='skip')


@pytest.fixture
def xfail_xp_backends(xp, request):
    """xfail_xp_backends(backend=None, reason=None, np_only=False, cpu_only=False, exceptions=None)

    xfail a decorated test for the provided backend, or xfail a category of backends.

    See ``skip_or_xfail_backends`` docstring for details. Note that, contrary to
    ``skip_or_xfail_backends``, the ``backend`` and ``reason`` arguments are optional
    single strings: this function only xfails a single backend at a time.
    To xfail multiple backends, provide multiple decorators.
    """  # noqa: E501
    if "xfail_xp_backends" not in request.keywords:
        return
    backends, kwargs = _backends_kwargs_from_request(request, skip_or_xfail='xfail')
    skip_or_xfail_xp_backends(xp, backends, kwargs, skip_or_xfail='xfail')


def skip_or_xfail_xp_backends(xp, backends, kwargs, skip_or_xfail='skip'):
    """
    Skip based on the ``skip_xp_backends`` or ``xfail_xp_backends`` marker.

    See the "Support for the array API standard" docs page for usage examples.

    Parameters
    ----------
    backends : tuple
        Backends to skip/xfail, e.g. ``("array_api_strict", "torch")``.
        These are overriden when ``np_only`` is ``True``, and are not
        necessary to provide for non-CPU backends when ``cpu_only`` is ``True``.
        For a custom reason to apply, you should pass a dict ``{'reason': '...'}``
        to a keyword matching the name of the backend.
    reason : str, optional
        A reason for the skip/xfail in the case of ``np_only=True``.
        If unprovided, a default reason is used. Note that it is not possible
        to specify a custom reason with ``cpu_only``.
    np_only : bool, optional
        When ``True``, the test is skipped/xfailed for all backends other
        than the default NumPy backend. There is no need to provide
        any ``backends`` in this case. To specify a reason, pass a
        value to ``reason``. Default: ``False``.
    cpu_only : bool, optional
        When ``True``, the test is skipped/xfailed on non-CPU devices.
        There is no need to provide any ``backends`` in this case,
        but any ``backends`` will also be skipped on the CPU.
        Default: ``False``.
    exceptions : list, optional
        A list of exceptions for use with ``cpu_only`` or ``np_only``.
        This should be provided when delegation is implemented for some,
        but not all, non-CPU/non-NumPy backends.
    skip_or_xfail : str
        ``'skip'`` to skip, ``'xfail'`` to xfail.
    """
    skip_or_xfail = getattr(pytest, skip_or_xfail)
    np_only = kwargs.get("np_only", False)
    cpu_only = kwargs.get("cpu_only", False)
    exceptions = kwargs.get("exceptions", [])

    if reasons := kwargs.get("reasons"):
        raise ValueError(f"provide a single `reason=` kwarg; got {reasons=} instead")

    # input validation
    if np_only and cpu_only:
        # np_only is a stricter subset of cpu_only
        cpu_only = False
    if exceptions and not (cpu_only or np_only):
        raise ValueError("`exceptions` is only valid alongside `cpu_only` or `np_only`")

    if np_only:
        reason = kwargs.get("reason", "do not run with non-NumPy backends.")
        if not isinstance(reason, str) and len(reason) > 1:
            raise ValueError("please provide a singleton `reason` "
                             "when using `np_only`")
        if xp.__name__ != 'numpy' and xp.__name__ not in exceptions:
            skip_or_xfail(reason=reason)
        return
    if cpu_only:
        reason = ("no array-agnostic implementation or delegation available "
                  "for this backend and device")
        exceptions = [] if exceptions is None else exceptions
        if SCIPY_ARRAY_API and SCIPY_DEVICE != 'cpu':
            if xp.__name__ == 'cupy' and 'cupy' not in exceptions:
                skip_or_xfail(reason=reason)
            elif xp.__name__ == 'torch' and 'torch' not in exceptions:
                if 'cpu' not in xp.empty(0).device.type:
                    skip_or_xfail(reason=reason)
            elif xp.__name__ == 'jax.numpy' and 'jax.numpy' not in exceptions:
                for d in xp.empty(0).devices():
                    if 'cpu' not in d.device_kind:
                        skip_or_xfail(reason=reason)

    if backends is not None:
        for i, backend in enumerate(backends):
            if xp.__name__ == backend:
                reason = kwargs[backend].get('reason')
                if not reason:
                    reason = f"do not run with array API backend: {backend}"

                skip_or_xfail(reason=reason)


# Following the approach of NumPy's conftest.py...
# Use a known and persistent tmpdir for hypothesis' caches, which
# can be automatically cleared by the OS or user.
hypothesis.configuration.set_hypothesis_home_dir(
    os.path.join(tempfile.gettempdir(), ".hypothesis")
)

# We register two custom profiles for SciPy - for details see
# https://hypothesis.readthedocs.io/en/latest/settings.html
# The first is designed for our own CI runs; the latter also
# forces determinism and is designed for use via scipy.test()
hypothesis.settings.register_profile(
    name="nondeterministic", deadline=None, print_blob=True,
)
hypothesis.settings.register_profile(
    name="deterministic",
    deadline=None, print_blob=True, database=None, derandomize=True,
    suppress_health_check=list(hypothesis.HealthCheck),
)

# Profile is currently set by environment variable `SCIPY_HYPOTHESIS_PROFILE`
# In the future, it would be good to work the choice into dev.py.
SCIPY_HYPOTHESIS_PROFILE = os.environ.get("SCIPY_HYPOTHESIS_PROFILE",
                                          "deterministic")
hypothesis.settings.load_profile(SCIPY_HYPOTHESIS_PROFILE)


############################################################################
# doctesting stuff

if HAVE_SCPDT:

    # FIXME: populate the dict once
    @contextmanager
    def warnings_errors_and_rng(test=None):
        """Temporarily turn (almost) all warnings to errors.

        Filter out known warnings which we allow.
        """
        known_warnings = dict()

        # these functions are known to emit "divide by zero" RuntimeWarnings
        divide_by_zero = [
            'scipy.linalg.norm', 'scipy.ndimage.center_of_mass',
        ]
        for name in divide_by_zero:
            known_warnings[name] = dict(category=RuntimeWarning,
                                        message='divide by zero')

        # Deprecated stuff in scipy.signal and elsewhere
        deprecated = [
            'scipy.signal.cwt', 'scipy.signal.morlet', 'scipy.signal.morlet2',
            'scipy.signal.ricker',
            'scipy.integrate.simpson',
            'scipy.interpolate.interp2d',
            'scipy.linalg.kron',
        ]
        for name in deprecated:
            known_warnings[name] = dict(category=DeprecationWarning)

        from scipy import integrate
        # the functions are known to emit IntegrationWarnings
        integration_w = ['scipy.special.ellip_normal',
                         'scipy.special.ellip_harm_2',
        ]
        for name in integration_w:
            known_warnings[name] = dict(category=integrate.IntegrationWarning,
                                        message='The occurrence of roundoff')

        # scipy.stats deliberately emits UserWarnings sometimes
        user_w = ['scipy.stats.anderson_ksamp', 'scipy.stats.kurtosistest',
                  'scipy.stats.normaltest', 'scipy.sparse.linalg.norm']
        for name in user_w:
            known_warnings[name] = dict(category=UserWarning)

        # additional one-off warnings to filter
        dct = {
            'scipy.sparse.linalg.norm':
                dict(category=UserWarning, message="Exited at iteration"),
            # tutorials
            'linalg.rst':
                dict(message='the matrix subclass is not',
                     category=PendingDeprecationWarning),
            'stats.rst':
                dict(message='The maximum number of subdivisions',
                     category=integrate.IntegrationWarning),
        }
        known_warnings.update(dct)

        # these legitimately emit warnings in examples
        legit = set('scipy.signal.normalize')

        # Now, the meat of the matter: filter warnings,
        # also control the random seed for each doctest.

        # XXX: this matches the refguide-check behavior, but is a tad strange:
        # makes sure that the seed the old-fashioned np.random* methods is
        # *NOT* reproducible but the new-style `default_rng()` *IS* repoducible.
        # Should these two be either both repro or both not repro?

        from scipy._lib._util import _fixed_default_rng
        import numpy as np
        with _fixed_default_rng():
            np.random.seed(None)
            with warnings.catch_warnings():
                if test and test.name in known_warnings:
                    warnings.filterwarnings('ignore',
                                            **known_warnings[test.name])
                    yield
                elif test and test.name in legit:
                    yield
                else:
                    warnings.simplefilter('error', Warning)
                    yield

    dt_config.user_context_mgr = warnings_errors_and_rng
    dt_config.skiplist = set([
        'scipy.linalg.LinAlgError',     # comes from numpy
        'scipy.fftpack.fftshift',       # fftpack stuff is also from numpy
        'scipy.fftpack.ifftshift',
        'scipy.fftpack.fftfreq',
        'scipy.special.sinc',           # sinc is from numpy
        'scipy.optimize.show_options',  # does not have much to doctest
        'scipy.signal.normalize',       # manipulates warnings (XXX temp skip)
        'scipy.sparse.linalg.norm',     # XXX temp skip
        # these below test things which inherit from np.ndarray
        # cross-ref https://github.com/numpy/numpy/issues/28019
        'scipy.io.matlab.MatlabObject.strides',
        'scipy.io.matlab.MatlabObject.dtype',
        'scipy.io.matlab.MatlabOpaque.dtype',
        'scipy.io.matlab.MatlabOpaque.strides',
        'scipy.io.matlab.MatlabFunction.strides',
        'scipy.io.matlab.MatlabFunction.dtype'
    ])

    # these are affected by NumPy 2.0 scalar repr: rely on string comparison
    if np.__version__ < "2":
        dt_config.skiplist.update(set([
            'scipy.io.hb_read',
            'scipy.io.hb_write',
            'scipy.sparse.csgraph.connected_components',
            'scipy.sparse.csgraph.depth_first_order',
            'scipy.sparse.csgraph.shortest_path',
            'scipy.sparse.csgraph.floyd_warshall',
            'scipy.sparse.csgraph.dijkstra',
            'scipy.sparse.csgraph.bellman_ford',
            'scipy.sparse.csgraph.johnson',
            'scipy.sparse.csgraph.yen',
            'scipy.sparse.csgraph.breadth_first_order',
            'scipy.sparse.csgraph.reverse_cuthill_mckee',
            'scipy.sparse.csgraph.structural_rank',
            'scipy.sparse.csgraph.construct_dist_matrix',
            'scipy.sparse.csgraph.reconstruct_path',
            'scipy.ndimage.value_indices',
            'scipy.stats.mstats.describe',
    ]))

    # help pytest collection a bit: these names are either private
    # (distributions), or just do not need doctesting.
    dt_config.pytest_extra_ignore = [
        "scipy.stats.distributions",
        "scipy.optimize.cython_optimize",
        "scipy.test",
        "scipy.show_config",
        # equivalent to "pytest --ignore=path/to/file"
        "scipy/special/_precompute",
        "scipy/interpolate/_interpnd_info.py",
        "scipy/_lib/array_api_compat",
        "scipy/_lib/highs",
        "scipy/_lib/unuran",
        "scipy/_lib/_gcutils.py",
        "scipy/_lib/doccer.py",
        "scipy/_lib/_uarray",
    ]

    dt_config.pytest_extra_xfail = {
        # name: reason
        "ND_regular_grid.rst": "ReST parser limitation",
        "extrapolation_examples.rst": "ReST parser limitation",
        "sampling_pinv.rst": "__cinit__ unexpected argument",
        "sampling_srou.rst": "nan in scalar_power",
        "probability_distributions.rst": "integration warning",
    }

    # tutorials
    dt_config.pseudocode = set(['integrate.nquad(func,'])
    dt_config.local_resources = {
        'io.rst': [
            "octave_a.mat",
            "octave_cells.mat",
            "octave_struct.mat"
        ]
    }

    dt_config.strict_check = True
############################################################################
